/*

  This file contains the state setup routines for all of the rendering
  methods used by Power Render and 3Dfx cards.

  THIS IS FOR ADVANCED USERS ONLY!

  Adding new state functions allows you to use different blending functions
  available on 3D cards.  You cannot add different triangle functionality
  such as anti-aliasing.  However, this can be used for adding phong
  highlights by adding alpha, or translucent gouraud shaded texture, etc.
  You must be careful that one state change will not effect another, such
  as leaving a blending function turned on when another method does not
  reset this state.

  To change an existing routine or add a new rendering type, first
  #include the prstate.h file.

  Second, copy one of the routines below into your own program and
  make modifications as necessary.

  Finally, call the PR_RegisterRenderMethod function to register
  the method with the rendering pipeline. It is defined as follows:

  PR_RegisterRenderMethod (int method_number, void (*MethodProc)(),
                PR_DWORD flags)

  The flags variable is TRUE or FALSE.  TRUE means the method uses
  alpha blending and triangles will be kept in a list and presorted
  before drawing the final image.  FALSE will draw the triangles
  immediately.

  Each state function has two sections:  setting up the texture and
  lighting when the ADDPRIV_setup_data variable is TRUE, and
  setting up the correct blending functions when the ADDPRIV_setup_state
  variable is TRUE.  The function will be called twice if using
  alpha blending (determined by the flags above), first setting up
  the data, and later setting up the blending functions when the
  PR_RenderFrame function is called.

  If alpha blending is not required, the function is called once and
  both sections are performed.

  The complete rendering method list is registered using a function called
  PR_InitializeRenderMethods at the bottom of this file.  You can rename
  this and use your own set of functions if you like.

*/


PR_DWORD AddAlphaFog1 (void)
{
PR_REAL fog;
PR_DWORD fogged;

  fog = 0.0f;
  fogged = 0;
  if (ADDPRIV_vptr1data->vz > PR_Settings.Fog_Far)
    {
     fog = 1.0f;
     fogged = 1;
    }
  else if (ADDPRIV_vptr1data->vz > PR_Settings.Fog_Near)
    {
     fog = (ADDPRIV_vptr1data->vz - PR_Settings.Fog_Near) * PR_Settings.Fog_Spread;
     fogged = 1;
    }              
  if (fog > 1.0f)
    fog = 1.0f;
  ADDPRIV_vptr1data->a *= 1.0f - fog;
  return fogged;
}

PR_DWORD AddAlphaFog2 (void)
{
PR_REAL fog;
PR_DWORD fogged;

  fog = 0.0f;
  fogged = 0;

  if (ADDPRIV_vptr2data->vz > PR_Settings.Fog_Far)
    {
     fog = 1.0f;
     fogged = 1;
    }
  else if (ADDPRIV_vptr2data->vz > PR_Settings.Fog_Near)
    {
     fog = (ADDPRIV_vptr2data->vz - PR_Settings.Fog_Near) * PR_Settings.Fog_Spread;
     fogged = 1;
    }
  if (fog > 1.0f)
    fog = 1.0f;
  ADDPRIV_vptr2data->a *= 1.0f - fog;

  return fogged;
}


PR_DWORD AddAlphaFog3 (void)
{
PR_REAL fog;
PR_DWORD fogged;

  fog = 0.0f;
  fogged = 0;

  if (ADDPRIV_vptr3data->vz > PR_Settings.Fog_Far)
    {
     fog = 1.0f;
     fogged = 1;
    }
  else if (ADDPRIV_vptr3data->vz > PR_Settings.Fog_Near)
    {
     fog = (ADDPRIV_vptr3data->vz - PR_Settings.Fog_Near) * PR_Settings.Fog_Spread;
     fogged = 1;
    }
  if (fog > 1.0f)
    fog = 1.0f;
  ADDPRIV_vptr3data->a *= 1.0f - fog;

  return fogged;
}





void PR_LightFunc_WIREFRAME (void) /* 0 */
{
  if (ADDPRIV_setup_state)
    {
     if (last_method != 1)
       {
        guColorCombineFunction (GR_COLORCOMBINE_ITRGB);

        grDepthBiasLevel (-3);
        grAlphaTestFunction (GR_CMP_ALWAYS);
        grAlphaBlendFunction (GR_BLEND_ONE, GR_BLEND_ZERO,
                              GR_BLEND_ONE, GR_BLEND_ZERO);
        last_method = 1;
       }
    }

  if (ADDPRIV_setup_data)
    {
     ADDPRIV_r = ADDPRIV_mat->r;
     ADDPRIV_g = ADDPRIV_mat->g;
     ADDPRIV_b = ADDPRIV_mat->b;

     ADDPRIV_vptr1data->r = ADDPRIV_vptr2data->r = ADDPRIV_vptr3data->r = ADDPRIV_r;
     ADDPRIV_vptr1data->a = ADDPRIV_mat->a;
     ADDPRIV_vptr1data->g = ADDPRIV_vptr2data->g = ADDPRIV_vptr3data->g = ADDPRIV_g;
     ADDPRIV_vptr2data->a = ADDPRIV_mat->a;
     ADDPRIV_vptr1data->b = ADDPRIV_vptr2data->b = ADDPRIV_vptr3data->b = ADDPRIV_b;
     ADDPRIV_vptr3data->a = ADDPRIV_mat->a;
    }
}




void PR_LightFunc_CFLAT (void) /* 1 */
{
PR_REAL i;

  if (ADDPRIV_setup_state)
    {
     if (last_method != 2)
       {
        guColorCombineFunction (GR_COLORCOMBINE_ITRGB);

        grDepthBiasLevel (0);
        grAlphaTestFunction (GR_CMP_ALWAYS);
        grAlphaBlendFunction (GR_BLEND_ONE, GR_BLEND_ZERO,
                              GR_BLEND_ONE, GR_BLEND_ZERO);

        last_method = 2;
       }
    }


  if (ADDPRIV_setup_data)
    {
     i = ADDPRIV_facedatareal->col[0];
     ADDPRIV_r = ADDPRIV_mat->r * i;
     ADDPRIV_g = ADDPRIV_mat->g * i;
     ADDPRIV_b = ADDPRIV_mat->b * i;

     ADDPRIV_vptr1data->r = ADDPRIV_vptr2data->r = ADDPRIV_vptr3data->r = ADDPRIV_r;
     ADDPRIV_vptr1data->a = ADDPRIV_mat->a;
     ADDPRIV_vptr1data->g = ADDPRIV_vptr2data->g = ADDPRIV_vptr3data->g = ADDPRIV_g;
     ADDPRIV_vptr2data->a = ADDPRIV_mat->a;
     ADDPRIV_vptr1data->b = ADDPRIV_vptr2data->b = ADDPRIV_vptr3data->b = ADDPRIV_b;
     ADDPRIV_vptr3data->a = ADDPRIV_mat->a;
    }
}


void PR_LightFunc_FLAT (void)  /* 2 */
{
PR_REAL i;

  if (ADDPRIV_setup_state)
    {
     if (last_method != 3)
       {
        guColorCombineFunction (GR_COLORCOMBINE_ITRGB);

        grDepthBiasLevel (0);
        grAlphaTestFunction (GR_CMP_ALWAYS);
        grAlphaBlendFunction (GR_BLEND_ONE, GR_BLEND_ZERO,
                              GR_BLEND_ONE, GR_BLEND_ZERO);
        last_method = 3;
       }
     }


  if (ADDPRIV_setup_data)
    {
     PR_WORKFACEFLAGS = !(ADDPRIV_face->flags & FFLAG_BACK_VISIBLE);
     PR_CalcFaceLighting3Dfx (ADDPRIV_face);

     i = ADDPRIV_facedatareal->col[0];          /* Add the precalced light */
     if (i > 0)
       {
        ADDPRIV_r += ADDPRIV_mat->r * i;
        ADDPRIV_g += ADDPRIV_mat->g * i;
        ADDPRIV_b += ADDPRIV_mat->b * i;
        CLAMP_RGB;
       }

     ADDPRIV_vptr1data->r = ADDPRIV_vptr2data->r = ADDPRIV_vptr3data->r = ADDPRIV_vptr1data->r = ADDPRIV_r;
     ADDPRIV_vptr1data->a = ADDPRIV_mat->a;
     ADDPRIV_vptr1data->g = ADDPRIV_vptr2data->g = ADDPRIV_vptr3data->g = ADDPRIV_vptr1data->g = ADDPRIV_g;
     ADDPRIV_vptr2data->a = ADDPRIV_mat->a;
     ADDPRIV_vptr1data->b = ADDPRIV_vptr2data->b = ADDPRIV_vptr3data->b = ADDPRIV_vptr1data->b = ADDPRIV_b;
     ADDPRIV_vptr3data->a = ADDPRIV_mat->a;
    }
}


void PR_LightFunc_CGOURAUD (void)  /* 3 */
{
PR_REAL i;

  if (ADDPRIV_setup_state)
    {
     if (last_method != 4)
       {
        guColorCombineFunction (GR_COLORCOMBINE_ITRGB);

        grDepthBiasLevel (0);
        grAlphaTestFunction (GR_CMP_ALWAYS);
        grAlphaBlendFunction (GR_BLEND_ONE, GR_BLEND_ZERO,
                              GR_BLEND_ONE, GR_BLEND_ZERO);
        last_method = 4;
       }
    }


  if (ADDPRIV_setup_data)
    {
     i = ADDPRIV_facedatareal->col[0];
     ADDPRIV_vptr1data->r = ADDPRIV_mat->r * i;
     ADDPRIV_vptr1data->g = ADDPRIV_mat->g * i;
     ADDPRIV_vptr1data->b = ADDPRIV_mat->b * i;
     ADDPRIV_vptr1data->a = ADDPRIV_mat->a;

     i = ADDPRIV_facedatareal->col[1];
     ADDPRIV_vptr2data->r = ADDPRIV_mat->r * i;
     ADDPRIV_vptr2data->g = ADDPRIV_mat->g * i;
     ADDPRIV_vptr2data->b = ADDPRIV_mat->b * i;
     ADDPRIV_vptr2data->a = ADDPRIV_mat->a;

     i = ADDPRIV_facedatareal->col[2];
     ADDPRIV_vptr3data->r = ADDPRIV_mat->r * i;
     ADDPRIV_vptr3data->g = ADDPRIV_mat->g * i;
     ADDPRIV_vptr3data->b = ADDPRIV_mat->b * i;
     ADDPRIV_vptr3data->a = ADDPRIV_mat->a;
    }
}




void PR_LightFunc_GOURAUD (void)  /* 4 */
{
PR_REAL i;


  if (ADDPRIV_setup_state)
    {
     if (last_method != 5)
       {
        guColorCombineFunction (GR_COLORCOMBINE_ITRGB);

        grDepthBiasLevel (0);
        grAlphaTestFunction (GR_CMP_ALWAYS);
        grAlphaBlendFunction (GR_BLEND_ONE, GR_BLEND_ZERO,
                              GR_BLEND_ONE, GR_BLEND_ZERO);
        last_method = 5;
       }
    }


  if (ADDPRIV_setup_data)
    {
     PR_WORKFACEFLAGS = !(ADDPRIV_face->flags & FFLAG_BACK_VISIBLE);

     if (ADDPRIV_vptr1data->dotproduct != ADDPRIV_material_num)
       {
        ADDPRIV_vptr1data->dotproduct = ADDPRIV_material_num;
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr1, ADDPRIV_vptr1data);

        i = ADDPRIV_facedatareal->col[0];          /* Add the precalced light */
        if (i > 0)
          {
           ADDPRIV_r += ADDPRIV_mat->r * i;
           ADDPRIV_g += ADDPRIV_mat->g * i;
           ADDPRIV_b += ADDPRIV_mat->b * i;
           CLAMP_RGB;
          }

        ADDPRIV_vptr1data->r = ADDPRIV_r;
        ADDPRIV_vptr1data->g = ADDPRIV_g;
        ADDPRIV_vptr1data->b = ADDPRIV_b;
        ADDPRIV_vptr1data->a = ADDPRIV_mat->a;
       }

     if (ADDPRIV_vptr2data->dotproduct != ADDPRIV_material_num)
       {
        ADDPRIV_vptr2data->dotproduct = ADDPRIV_material_num;
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr2, ADDPRIV_vptr2data);

        i = ADDPRIV_facedatareal->col[1];          /* Add the precalced light */
        if (i > 0)
          {
           ADDPRIV_r += ADDPRIV_mat->r * i;
           ADDPRIV_g += ADDPRIV_mat->g * i;
           ADDPRIV_b += ADDPRIV_mat->b * i;
           CLAMP_RGB;
          }

        ADDPRIV_vptr2data->r = ADDPRIV_r;
        ADDPRIV_vptr2data->g = ADDPRIV_g;
        ADDPRIV_vptr2data->b = ADDPRIV_b;
        ADDPRIV_vptr2data->a = ADDPRIV_mat->a;
       }

     if (ADDPRIV_vptr3data->dotproduct != ADDPRIV_material_num)
       {
        ADDPRIV_vptr3data->dotproduct = ADDPRIV_material_num;
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr3, ADDPRIV_vptr3data);

        i = ADDPRIV_facedatareal->col[2];          /* Add the precalced light */
        if (i > 0)
          {
           ADDPRIV_r += ADDPRIV_mat->r * i;
           ADDPRIV_g += ADDPRIV_mat->g * i;
           ADDPRIV_b += ADDPRIV_mat->b * i;
           CLAMP_RGB;
          }

        ADDPRIV_vptr3data->r = ADDPRIV_r;
        ADDPRIV_vptr3data->g = ADDPRIV_g;
        ADDPRIV_vptr3data->b = ADDPRIV_b;
        ADDPRIV_vptr3data->a = ADDPRIV_mat->a;
       }
    }
}


void PR_LightFunc_DECAL (void)    /* 5 */
{
PR_FACE_DATA_REAL *faceptr;

  if (ADDPRIV_setup_state)
    {
     if (last_material != ADDPRIV_mat)
       {
        PR_SelectTexture (ADDPRIV_mat->texture_number);
        last_material = ADDPRIV_mat;
       }

     if (last_method != 8)
       {
        guColorCombineFunction (GR_COLORCOMBINE_DECAL_TEXTURE);
        grTexCombineFunction (PR_CurrentTMU, GR_TEXTURECOMBINE_DECAL);

        grDepthBiasLevel (0);

        grAlphaTestFunction (GR_CMP_ALWAYS);
        grAlphaBlendFunction (GR_BLEND_ONE, GR_BLEND_ZERO,
                              GR_BLEND_ONE, GR_BLEND_ZERO);
        last_method = 8;
       }
    }


  if (ADDPRIV_setup_data)
    {
     faceptr = (PR_FACE_DATA_REAL *)&ADDPRIV_face->face_data;
     SET_TMAP(ADDPRIV_mat, ADDPRIV_vptr1data, ADDPRIV_vptr2data, ADDPRIV_vptr3data, ADDPRIV_vptr1, ADDPRIV_vptr2, ADDPRIV_vptr3, ADDPRIV_vptr1data,ADDPRIV_vptr2data,ADDPRIV_vptr3data,faceptr);
     ADDPRIV_vptr1data->a = ADDPRIV_mat->a;
     ADDPRIV_vptr2data->a = ADDPRIV_mat->a;
     ADDPRIV_vptr3data->a = ADDPRIV_mat->a;
    }
}


void PR_LightFunc_XDECAL (void)    /* 6 */
{
PR_FACE_DATA_REAL *faceptr;

  if (ADDPRIV_setup_state)
    {
     if (last_material != ADDPRIV_mat)
       {
        PR_SelectTexture (ADDPRIV_mat->texture_number);
        last_material = ADDPRIV_mat;
       }

     if (last_method != 10)
       {
        guColorCombineFunction (GR_COLORCOMBINE_DECAL_TEXTURE);
        grTexCombineFunction (PR_CurrentTMU, GR_TEXTURECOMBINE_DECAL);

        grDepthBiasLevel (0);
        grAlphaTestReferenceValue (1);
        grAlphaTestFunction (GR_CMP_GREATER);

        grAlphaCombine(GR_COMBINE_FUNCTION_BLEND,
                       GR_COMBINE_FACTOR_ONE,
                       GR_COMBINE_LOCAL_NONE,
                       GR_COMBINE_OTHER_TEXTURE,
                       FXFALSE );
        grAlphaBlendFunction (GR_BLEND_SRC_ALPHA, GR_BLEND_ONE_MINUS_SRC_ALPHA,
                              GR_BLEND_ONE, GR_BLEND_ZERO);
        last_method = 10;
       }
    }


  if (ADDPRIV_setup_data)
    {
     faceptr = (PR_FACE_DATA_REAL *)&ADDPRIV_face->face_data;
     SET_TMAP(ADDPRIV_mat, ADDPRIV_vptr1data, ADDPRIV_vptr2data, ADDPRIV_vptr3data, ADDPRIV_vptr1, ADDPRIV_vptr2, ADDPRIV_vptr3, ADDPRIV_vptr1data,ADDPRIV_vptr2data,ADDPRIV_vptr3data,faceptr);
     ADDPRIV_vptr1data->a = ADDPRIV_mat->a;
     ADDPRIV_vptr2data->a = ADDPRIV_mat->a;
     ADDPRIV_vptr3data->a = ADDPRIV_mat->a;
    }
}


void PR_LightFunc_CFLATTEXT (void)  /* 7 */
{
PR_REAL i;
PR_FACE_DATA_REAL *faceptr;

  if (ADDPRIV_setup_state)
    {
     if (last_material != ADDPRIV_mat)
       {
        PR_SelectTexture (ADDPRIV_mat->texture_number);
        last_material = ADDPRIV_mat;
       }

     if (last_method != 14)
       {
        guColorCombineFunction (GR_COLORCOMBINE_TEXTURE_TIMES_ITRGB);
        grTexCombineFunction (PR_CurrentTMU, GR_TEXTURECOMBINE_DECAL);

        grDepthBiasLevel (0);
        grAlphaBlendFunction (GR_BLEND_ONE, GR_BLEND_ZERO,
                              GR_BLEND_ONE, GR_BLEND_ZERO);
        grAlphaTestFunction (GR_CMP_ALWAYS);
        last_method = 14;
       }
    }


  if (ADDPRIV_setup_data)
    {
     i = ADDPRIV_facedatareal->col[0];
     ADDPRIV_r = ADDPRIV_mat->r * i;
     ADDPRIV_g = ADDPRIV_mat->g * i;
     ADDPRIV_b = ADDPRIV_mat->b * i;

     ADDPRIV_vptr1data->r = ADDPRIV_vptr2data->r = ADDPRIV_vptr3data->r = ADDPRIV_r;
     ADDPRIV_vptr1data->a = ADDPRIV_mat->a;
     ADDPRIV_vptr1data->g = ADDPRIV_vptr2data->g = ADDPRIV_vptr3data->g = ADDPRIV_g;
     ADDPRIV_vptr2data->a = ADDPRIV_mat->a;
     ADDPRIV_vptr1data->b = ADDPRIV_vptr2data->b = ADDPRIV_vptr3data->b = ADDPRIV_b;
     ADDPRIV_vptr3data->a = ADDPRIV_mat->a;

     faceptr = (PR_FACE_DATA_REAL *)&ADDPRIV_face->face_data;
     SET_TMAP(ADDPRIV_mat, ADDPRIV_vptr1data, ADDPRIV_vptr2data, ADDPRIV_vptr3data, ADDPRIV_vptr1, ADDPRIV_vptr2, ADDPRIV_vptr3, ADDPRIV_vptr1data,ADDPRIV_vptr2data,ADDPRIV_vptr3data,faceptr);
    }
}


void PR_LightFunc_XCFLATTEXT (void)  /* 8 */
{
PR_REAL i;
PR_FACE_DATA_REAL *faceptr;

  if (ADDPRIV_setup_state)
    {
     if (last_material != ADDPRIV_mat)
       {
        PR_SelectTexture (ADDPRIV_mat->texture_number);
        last_material = ADDPRIV_mat;
       }

     if (last_method != 16)
       {
        guColorCombineFunction (GR_COLORCOMBINE_TEXTURE_TIMES_ITRGB);
        grTexCombineFunction (PR_CurrentTMU, GR_TEXTURECOMBINE_DECAL);

        grDepthBiasLevel (0);
        grAlphaTestReferenceValue (1);
        grAlphaTestFunction (GR_CMP_GREATER);

        grAlphaCombine(GR_COMBINE_FUNCTION_BLEND,
                       GR_COMBINE_FACTOR_ONE,
                       GR_COMBINE_LOCAL_NONE,
                       GR_COMBINE_OTHER_TEXTURE,
                       FXFALSE );
        grAlphaBlendFunction (GR_BLEND_SRC_ALPHA, GR_BLEND_ONE_MINUS_SRC_ALPHA,
                              GR_BLEND_ONE, GR_BLEND_ZERO);
                           
        last_method = 16;
       }
    }


  if (ADDPRIV_setup_data)
    {
     i = ADDPRIV_facedatareal->col[0];
     ADDPRIV_r = ADDPRIV_mat->r * i;
     ADDPRIV_g = ADDPRIV_mat->g * i;
     ADDPRIV_b = ADDPRIV_mat->b * i;

     ADDPRIV_vptr1data->r = ADDPRIV_vptr2data->r = ADDPRIV_vptr3data->r = ADDPRIV_r;
     ADDPRIV_vptr1data->a = ADDPRIV_mat->a;
     ADDPRIV_vptr1data->g = ADDPRIV_vptr2data->g = ADDPRIV_vptr3data->g = ADDPRIV_g;
     ADDPRIV_vptr2data->a = ADDPRIV_mat->a;
     ADDPRIV_vptr1data->b = ADDPRIV_vptr2data->b = ADDPRIV_vptr3data->b = ADDPRIV_b;
     ADDPRIV_vptr3data->a = ADDPRIV_mat->a;

     faceptr = (PR_FACE_DATA_REAL *)&ADDPRIV_face->face_data;
     SET_TMAP(ADDPRIV_mat, ADDPRIV_vptr1data, ADDPRIV_vptr2data, ADDPRIV_vptr3data, ADDPRIV_vptr1, ADDPRIV_vptr2, ADDPRIV_vptr3, ADDPRIV_vptr1data,ADDPRIV_vptr2data,ADDPRIV_vptr3data,faceptr);
    }
}


void PR_LightFunc_FLATTEXT (void)   /* 9 */
{
PR_FACE_DATA_REAL *faceptr;
PR_REAL i;

  if (ADDPRIV_setup_state)
    {
     if (last_material != ADDPRIV_mat)
       {
        PR_SelectTexture (ADDPRIV_mat->texture_number);
        last_material = ADDPRIV_mat;
       }

     if (last_method != 18)
       {
        guColorCombineFunction (GR_COLORCOMBINE_TEXTURE_TIMES_ITRGB);
        grTexCombineFunction (PR_CurrentTMU, GR_TEXTURECOMBINE_DECAL);

        grDepthBiasLevel (0);
        grAlphaBlendFunction (GR_BLEND_ONE, GR_BLEND_ZERO,
                              GR_BLEND_ONE, GR_BLEND_ZERO);
        grAlphaTestFunction (GR_CMP_ALWAYS);
        last_method = 18;
       }
    }


  if (ADDPRIV_setup_data)
    {
     PR_WORKFACEFLAGS = !(ADDPRIV_face->flags & FFLAG_BACK_VISIBLE);
     PR_CalcFaceLighting3Dfx (ADDPRIV_face);

     i = ADDPRIV_facedatareal->col[0];          /* Add the precalced light */
     if (i > 0)
       {
        ADDPRIV_r += ADDPRIV_mat->r * i;
        ADDPRIV_g += ADDPRIV_mat->g * i;
        ADDPRIV_b += ADDPRIV_mat->b * i;
        CLAMP_RGB;
       }

     ADDPRIV_vptr1data->r = ADDPRIV_vptr2data->r = ADDPRIV_vptr3data->r = ADDPRIV_r;
     ADDPRIV_vptr1data->a = ADDPRIV_mat->a;
     ADDPRIV_vptr1data->g = ADDPRIV_vptr2data->g = ADDPRIV_vptr3data->g = ADDPRIV_g;
     ADDPRIV_vptr2data->a = ADDPRIV_mat->a;
     ADDPRIV_vptr1data->b = ADDPRIV_vptr2data->b = ADDPRIV_vptr3data->b = ADDPRIV_b;
     ADDPRIV_vptr3data->a = ADDPRIV_mat->a;

     faceptr = (PR_FACE_DATA_REAL *)&ADDPRIV_face->face_data;
     SET_TMAP(ADDPRIV_mat, ADDPRIV_vptr1data, ADDPRIV_vptr2data, ADDPRIV_vptr3data, ADDPRIV_vptr1, ADDPRIV_vptr2, ADDPRIV_vptr3, ADDPRIV_vptr1data,ADDPRIV_vptr2data,ADDPRIV_vptr3data,faceptr);
    }
}



void PR_LightFunc_XFLATTEXT (void)   /* 10 */
{
PR_FACE_DATA_REAL *faceptr;
PR_REAL i;

  if (ADDPRIV_setup_state)
    {
     if (last_material != ADDPRIV_mat)
       {
        PR_SelectTexture (ADDPRIV_mat->texture_number);
        last_material = ADDPRIV_mat;
       }

     if (last_method != 20)
       {
        guColorCombineFunction (GR_COLORCOMBINE_TEXTURE_TIMES_ITRGB);
        grTexCombineFunction (PR_CurrentTMU, GR_TEXTURECOMBINE_DECAL);

        grAlphaTestReferenceValue (1);
        grAlphaTestFunction (GR_CMP_GREATER);

        grDepthBiasLevel (0);
        grAlphaCombine(GR_COMBINE_FUNCTION_BLEND,
                       GR_COMBINE_FACTOR_ONE,
                       GR_COMBINE_LOCAL_NONE,
                       GR_COMBINE_OTHER_TEXTURE,
                       FXFALSE );
        grAlphaBlendFunction (GR_BLEND_SRC_ALPHA, GR_BLEND_ONE_MINUS_SRC_ALPHA,
                              GR_BLEND_ONE, GR_BLEND_ZERO);
                           
        last_method = 20;
       }
    }


  if (ADDPRIV_setup_data)
    {
     PR_WORKFACEFLAGS = !(ADDPRIV_face->flags & FFLAG_BACK_VISIBLE);
     PR_CalcFaceLighting3Dfx (ADDPRIV_face);

     i = ADDPRIV_facedatareal->col[0];          /* Add the precalced light */
     if (i > 0)
       {
        ADDPRIV_r += ADDPRIV_mat->r * i;
        ADDPRIV_g += ADDPRIV_mat->g * i;
        ADDPRIV_b += ADDPRIV_mat->b * i;
        CLAMP_RGB;
       }

     ADDPRIV_vptr1data->r = ADDPRIV_vptr2data->r = ADDPRIV_vptr3data->r = ADDPRIV_r;
     ADDPRIV_vptr1data->a = ADDPRIV_mat->a;
     ADDPRIV_vptr1data->g = ADDPRIV_vptr2data->g = ADDPRIV_vptr3data->g = ADDPRIV_g;
     ADDPRIV_vptr2data->a = ADDPRIV_mat->a;
     ADDPRIV_vptr1data->b = ADDPRIV_vptr2data->b = ADDPRIV_vptr3data->b = ADDPRIV_b;
     ADDPRIV_vptr3data->a = ADDPRIV_mat->a;
 
     faceptr = (PR_FACE_DATA_REAL *)&ADDPRIV_face->face_data;
     SET_TMAP(ADDPRIV_mat, ADDPRIV_vptr1data, ADDPRIV_vptr2data, ADDPRIV_vptr3data, ADDPRIV_vptr1, ADDPRIV_vptr2, ADDPRIV_vptr3, ADDPRIV_vptr1data,ADDPRIV_vptr2data,ADDPRIV_vptr3data,faceptr);
    }
}




void PR_LightFunc_CGOURAUDTEXT (void)  /* 11 */
{
PR_FACE_DATA_REAL *faceptr;
PR_REAL i;

  if (ADDPRIV_setup_state)
    {
     if (last_material != ADDPRIV_mat)
       {
        PR_SelectTexture (ADDPRIV_mat->texture_number);
        last_material = ADDPRIV_mat;
       }

     if (last_method != 26)
       {
        guColorCombineFunction (GR_COLORCOMBINE_TEXTURE_TIMES_ITRGB);
        grTexCombineFunction (PR_CurrentTMU, GR_TEXTURECOMBINE_DECAL);

        grDepthBiasLevel (0);
        grAlphaBlendFunction (GR_BLEND_ONE, GR_BLEND_ZERO,
                              GR_BLEND_ONE, GR_BLEND_ZERO);
        grAlphaTestFunction (GR_CMP_ALWAYS);
        last_method = 26;
       }
    }


  if (ADDPRIV_setup_data)
    {
     i = ADDPRIV_facedatareal->col[0];
     ADDPRIV_vptr1data->r = ADDPRIV_mat->r * i;
     ADDPRIV_vptr1data->g = ADDPRIV_mat->g * i;
     ADDPRIV_vptr1data->b = ADDPRIV_mat->b * i;
     ADDPRIV_vptr1data->a = ADDPRIV_mat->a;

     i = ADDPRIV_facedatareal->col[1];
     ADDPRIV_vptr2data->r = ADDPRIV_mat->r * i;
     ADDPRIV_vptr2data->g = ADDPRIV_mat->g * i;
     ADDPRIV_vptr2data->b = ADDPRIV_mat->b * i;
     ADDPRIV_vptr2data->a = ADDPRIV_mat->a;

     i = ADDPRIV_facedatareal->col[2];
     ADDPRIV_vptr3data->r = ADDPRIV_mat->r * i;
     ADDPRIV_vptr3data->g = ADDPRIV_mat->g * i;
     ADDPRIV_vptr3data->b = ADDPRIV_mat->b * i;
     ADDPRIV_vptr3data->a = ADDPRIV_mat->a;

     faceptr = (PR_FACE_DATA_REAL *)&ADDPRIV_face->face_data;
     SET_TMAP(ADDPRIV_mat, ADDPRIV_vptr1data, ADDPRIV_vptr2data, ADDPRIV_vptr3data, ADDPRIV_vptr1, ADDPRIV_vptr2, ADDPRIV_vptr3, ADDPRIV_vptr1data,ADDPRIV_vptr2data,ADDPRIV_vptr3data,faceptr);
    }
}


void PR_LightFunc_XCGOURAUDTEXT (void)  /* 12 */
{
PR_FACE_DATA_REAL *faceptr;
PR_REAL i;

  if (ADDPRIV_setup_state)
    {
     if (last_material != ADDPRIV_mat)
       {
        PR_SelectTexture (ADDPRIV_mat->texture_number);
        last_material = ADDPRIV_mat;
       }

     if (last_method != 28)
       {
        guColorCombineFunction (GR_COLORCOMBINE_TEXTURE_TIMES_ITRGB);
        grTexCombineFunction (PR_CurrentTMU, GR_TEXTURECOMBINE_DECAL);

        grDepthBiasLevel (0);
        grAlphaTestReferenceValue (1);
        grAlphaTestFunction (GR_CMP_GREATER);

        grAlphaCombine(GR_COMBINE_FUNCTION_BLEND,
                       GR_COMBINE_FACTOR_ONE,
                       GR_COMBINE_LOCAL_NONE,
                       GR_COMBINE_OTHER_TEXTURE,
                       FXFALSE );
        grAlphaBlendFunction (GR_BLEND_SRC_ALPHA, GR_BLEND_ONE_MINUS_SRC_ALPHA,
                              GR_BLEND_ONE, GR_BLEND_ZERO);
        last_method = 28;
       }
    }


  if (ADDPRIV_setup_data)
    {
     i = ADDPRIV_facedatareal->col[0];
     ADDPRIV_vptr1data->r = ADDPRIV_mat->r * i;
     ADDPRIV_vptr1data->g = ADDPRIV_mat->g * i;
     ADDPRIV_vptr1data->b = ADDPRIV_mat->b * i;
     ADDPRIV_vptr1data->a = ADDPRIV_mat->a;

     i = ADDPRIV_facedatareal->col[1];
     ADDPRIV_vptr2data->r = ADDPRIV_mat->r * i;
     ADDPRIV_vptr2data->g = ADDPRIV_mat->g * i;
     ADDPRIV_vptr2data->b = ADDPRIV_mat->b * i;
     ADDPRIV_vptr2data->a = ADDPRIV_mat->a;

     i = ADDPRIV_facedatareal->col[2];
     ADDPRIV_vptr3data->r = ADDPRIV_mat->r * i;
     ADDPRIV_vptr3data->g = ADDPRIV_mat->g * i;
     ADDPRIV_vptr3data->b = ADDPRIV_mat->b * i;
     ADDPRIV_vptr3data->a = ADDPRIV_mat->a;

     faceptr = (PR_FACE_DATA_REAL *)&ADDPRIV_face->face_data;
     SET_TMAP(ADDPRIV_mat, ADDPRIV_vptr1data, ADDPRIV_vptr2data, ADDPRIV_vptr3data, ADDPRIV_vptr1, ADDPRIV_vptr2, ADDPRIV_vptr3, ADDPRIV_vptr1data,ADDPRIV_vptr2data,ADDPRIV_vptr3data,faceptr);
    }
}




void PR_LightFunc_GOURAUDTEXT (void)  /* 13 */
{
PR_FACE_DATA_REAL *faceptr;
PR_REAL i;


  if (ADDPRIV_setup_state)
    {
     if (last_material != ADDPRIV_mat)
       {
        PR_SelectTexture (ADDPRIV_mat->texture_number);
        last_material = ADDPRIV_mat;
       }

     if (last_method != 32)
       {
        guColorCombineFunction (GR_COLORCOMBINE_TEXTURE_TIMES_ITRGB);
        grTexCombineFunction (PR_CurrentTMU, GR_TEXTURECOMBINE_DECAL);

        grDepthBiasLevel (0);
        grAlphaBlendFunction (GR_BLEND_ONE, GR_BLEND_ZERO,
                              GR_BLEND_ONE, GR_BLEND_ZERO);
        grAlphaTestFunction (GR_CMP_ALWAYS);

        last_method = 32;
       }
    }
   

  if (ADDPRIV_setup_data)
    {
     PR_WORKFACEFLAGS = !(ADDPRIV_face->flags & FFLAG_BACK_VISIBLE);
     if (ADDPRIV_vptr1data->dotproduct != ADDPRIV_material_num)
       {
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr1, ADDPRIV_vptr1data);

        i = ADDPRIV_facedatareal->col[0];          /* Add the precalced light */
        if (i > 0)
          {
           ADDPRIV_r += ADDPRIV_mat->r * i;
           ADDPRIV_g += ADDPRIV_mat->g * i;
           ADDPRIV_b += ADDPRIV_mat->b * i;
           CLAMP_RGB;
           ADDPRIV_vptr1data->dotproduct = -1;
          }
        else
          ADDPRIV_vptr1data->dotproduct = ADDPRIV_material_num;
   
        ADDPRIV_vptr1data->r = ADDPRIV_r;
        ADDPRIV_vptr1data->g = ADDPRIV_g;
        ADDPRIV_vptr1data->b = ADDPRIV_b;
        ADDPRIV_vptr1data->a = ADDPRIV_mat->a;
       }

     if (ADDPRIV_vptr2data->dotproduct != ADDPRIV_material_num)
       {
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr2, ADDPRIV_vptr2data);
   
        i = ADDPRIV_facedatareal->col[1];          /* Add the precalced light */
        if (i > 0)
          {
           ADDPRIV_r += ADDPRIV_mat->r * i;
           ADDPRIV_g += ADDPRIV_mat->g * i;
           ADDPRIV_b += ADDPRIV_mat->b * i;
           CLAMP_RGB;
           ADDPRIV_vptr2data->dotproduct = -1;
          }
        else
          ADDPRIV_vptr2data->dotproduct = ADDPRIV_material_num;
   
        ADDPRIV_vptr2data->r = ADDPRIV_r;
        ADDPRIV_vptr2data->g = ADDPRIV_g;
        ADDPRIV_vptr2data->b = ADDPRIV_b;
        ADDPRIV_vptr2data->a = ADDPRIV_mat->a;
       }

     if (ADDPRIV_vptr3data->dotproduct != ADDPRIV_material_num)
       {
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr3, ADDPRIV_vptr3data);
   
        i = ADDPRIV_facedatareal->col[2];          /* Add the precalced light */
        if (i > 0)
          {
           ADDPRIV_r += ADDPRIV_mat->r * i;
           ADDPRIV_g += ADDPRIV_mat->g * i;
           ADDPRIV_b += ADDPRIV_mat->b * i;
           CLAMP_RGB;
           ADDPRIV_vptr3data->dotproduct = -1;
          }
        else
          ADDPRIV_vptr3data->dotproduct = ADDPRIV_material_num;
   
        ADDPRIV_vptr3data->r = ADDPRIV_r;
        ADDPRIV_vptr3data->g = ADDPRIV_g;
        ADDPRIV_vptr3data->b = ADDPRIV_b;
        ADDPRIV_vptr3data->a = ADDPRIV_mat->a;
       }

     faceptr = (PR_FACE_DATA_REAL *)&ADDPRIV_face->face_data;
     SET_TMAP(ADDPRIV_mat, ADDPRIV_vptr1data, ADDPRIV_vptr2data, ADDPRIV_vptr3data, ADDPRIV_vptr1, ADDPRIV_vptr2, ADDPRIV_vptr3, ADDPRIV_vptr1data,ADDPRIV_vptr2data,ADDPRIV_vptr3data,faceptr);
    }
}




void PR_LightFunc_XGOURAUDTEXT (void)  /* 14 */
{
PR_FACE_DATA_REAL *faceptr;
PR_REAL i;

  if (ADDPRIV_setup_state)
    {
     if (last_material != ADDPRIV_mat)
       {
        PR_SelectTexture (ADDPRIV_mat->texture_number);
        last_material = ADDPRIV_mat;
       }

     if (last_method != 34)
       {
        guColorCombineFunction (GR_COLORCOMBINE_TEXTURE_TIMES_ITRGB);
        grTexCombineFunction (PR_CurrentTMU, GR_TEXTURECOMBINE_DECAL);

        grDepthBiasLevel (0);
        grAlphaTestReferenceValue (1);
        grAlphaTestFunction (GR_CMP_GREATER);

        grAlphaCombine(GR_COMBINE_FUNCTION_BLEND,
                       GR_COMBINE_FACTOR_ONE,
                       GR_COMBINE_LOCAL_NONE,
                       GR_COMBINE_OTHER_TEXTURE,
                       FXFALSE );
        grAlphaBlendFunction (GR_BLEND_SRC_ALPHA, GR_BLEND_ONE_MINUS_SRC_ALPHA,
                              GR_BLEND_ONE, GR_BLEND_ZERO);

        last_method = 34;
       }
    }


  if (ADDPRIV_setup_data)
    {
     PR_WORKFACEFLAGS = !(ADDPRIV_face->flags & FFLAG_BACK_VISIBLE);
     if (ADDPRIV_vptr1data->dotproduct != ADDPRIV_material_num)
       {
        ADDPRIV_vptr1data->dotproduct = ADDPRIV_material_num;
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr1, ADDPRIV_vptr1data);

        i = ADDPRIV_facedatareal->col[0];          /* Add the precalced light */
        if (i > 0)
          {
           ADDPRIV_r += ADDPRIV_mat->r * i;
           ADDPRIV_g += ADDPRIV_mat->g * i;
           ADDPRIV_b += ADDPRIV_mat->b * i;
           CLAMP_RGB;
          }
        ADDPRIV_vptr1data->r = ADDPRIV_r;
        ADDPRIV_vptr1data->g = ADDPRIV_g;
        ADDPRIV_vptr1data->b = ADDPRIV_b;
        ADDPRIV_vptr1data->a = ADDPRIV_mat->a;
       }


     if (ADDPRIV_vptr2data->dotproduct != ADDPRIV_material_num)
       {
        ADDPRIV_vptr2data->dotproduct = ADDPRIV_material_num;
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr2, ADDPRIV_vptr2data);
   
        i = ADDPRIV_facedatareal->col[1];          /* Add the precalced light */
        if (i > 0)
          {
           ADDPRIV_r += ADDPRIV_mat->r * i;
           ADDPRIV_g += ADDPRIV_mat->g * i;
           ADDPRIV_b += ADDPRIV_mat->b * i;
           CLAMP_RGB;
          }
        ADDPRIV_vptr2data->r = ADDPRIV_r;
        ADDPRIV_vptr2data->g = ADDPRIV_g;
        ADDPRIV_vptr2data->b = ADDPRIV_b;
        ADDPRIV_vptr2data->a = ADDPRIV_mat->a;
       }

     if (ADDPRIV_vptr3data->dotproduct != ADDPRIV_material_num)
       {
        ADDPRIV_vptr3data->dotproduct = ADDPRIV_material_num;
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr3, ADDPRIV_vptr3data);
   
        i = ADDPRIV_facedatareal->col[2];          /* Add the precalced light */
        if (i > 0)
          {
           ADDPRIV_r += ADDPRIV_mat->r * i;
           ADDPRIV_g += ADDPRIV_mat->g * i;
           ADDPRIV_b += ADDPRIV_mat->b * i;
           CLAMP_RGB;
          }
        ADDPRIV_vptr3data->r = ADDPRIV_r;
        ADDPRIV_vptr3data->g = ADDPRIV_g;
        ADDPRIV_vptr3data->b = ADDPRIV_b;
        ADDPRIV_vptr3data->a = ADDPRIV_mat->a;
       }

     faceptr = (PR_FACE_DATA_REAL *)&ADDPRIV_face->face_data;
     SET_TMAP(ADDPRIV_mat, ADDPRIV_vptr1data, ADDPRIV_vptr2data, ADDPRIV_vptr3data, ADDPRIV_vptr1, ADDPRIV_vptr2, ADDPRIV_vptr3, ADDPRIV_vptr1data,ADDPRIV_vptr2data,ADDPRIV_vptr3data,faceptr);
    }
}



void PR_LightFunc_TRANSTEXT (void)    /* 15 */
{
PR_FACE_DATA_REAL *faceptr;

  if (ADDPRIV_setup_state)
    {
     if (last_material != ADDPRIV_mat)
       {
        PR_SelectTexture (ADDPRIV_mat->texture_number);
        last_material = ADDPRIV_mat;
       }

     if (last_method != 38)
       {
        guColorCombineFunction (GR_COLORCOMBINE_DECAL_TEXTURE);
        grTexCombineFunction (PR_CurrentTMU, GR_TEXTURECOMBINE_DECAL);

        grDepthBiasLevel (0);
        guAlphaSource( GR_ALPHASOURCE_ITERATED_ALPHA);
        grAlphaBlendFunction (GR_BLEND_SRC_ALPHA, GR_BLEND_ONE_MINUS_SRC_ALPHA, GR_BLEND_ONE, GR_BLEND_ZERO);
        grAlphaTestFunction(GR_CMP_ALWAYS);
        last_method = 38;
       }
    }


  if (ADDPRIV_setup_data)
    {
     ADDPRIV_vptr1data->a = ADDPRIV_mat->a;
     ADDPRIV_vptr2data->a = ADDPRIV_mat->a;
     ADDPRIV_vptr3data->a = ADDPRIV_mat->a;

     faceptr = (PR_FACE_DATA_REAL *)&ADDPRIV_face->face_data;
     SET_TMAP(ADDPRIV_mat, ADDPRIV_vptr1data, ADDPRIV_vptr2data, ADDPRIV_vptr3data, ADDPRIV_vptr1, ADDPRIV_vptr2, ADDPRIV_vptr3, ADDPRIV_vptr1data,ADDPRIV_vptr2data,ADDPRIV_vptr3data,faceptr);
    }
}


void PR_LightFunc_XTRANSTEXT (void)    /* 16 */
{
PR_FACE_DATA_REAL *faceptr;

  if (ADDPRIV_setup_state)
    {
     if (last_material != ADDPRIV_mat)
       {
        PR_SelectTexture (ADDPRIV_mat->texture_number);
        last_material = ADDPRIV_mat;
       }

     if (last_method != 40)
       {
        guColorCombineFunction (GR_COLORCOMBINE_DECAL_TEXTURE);
        grTexCombineFunction (PR_CurrentTMU, GR_TEXTURECOMBINE_DECAL);
   
        grDepthBiasLevel (0);
        grAlphaTestReferenceValue (1);
        grAlphaTestFunction (GR_CMP_GREATER);
   
        grAlphaCombine(GR_COMBINE_FUNCTION_BLEND,
                       GR_COMBINE_FACTOR_ONE,
                       GR_COMBINE_LOCAL_NONE,
                       GR_COMBINE_OTHER_TEXTURE,
                       FXFALSE );
        guAlphaSource (GR_ALPHASOURCE_TEXTURE_ALPHA_TIMES_ITERATED_ALPHA);
        grAlphaBlendFunction (GR_BLEND_SRC_ALPHA, GR_BLEND_ONE_MINUS_SRC_ALPHA, GR_BLEND_ONE, GR_BLEND_ZERO );
        last_method = 40;
       }
    }


  if (ADDPRIV_setup_data)
    {
     ADDPRIV_vptr1data->a = ADDPRIV_mat->a;
     ADDPRIV_vptr2data->a = ADDPRIV_mat->a;
     ADDPRIV_vptr3data->a = ADDPRIV_mat->a;

     faceptr = (PR_FACE_DATA_REAL *)&ADDPRIV_face->face_data;
     SET_TMAP(ADDPRIV_mat, ADDPRIV_vptr1data, ADDPRIV_vptr2data, ADDPRIV_vptr3data, ADDPRIV_vptr1, ADDPRIV_vptr2, ADDPRIV_vptr3, ADDPRIV_vptr1data,ADDPRIV_vptr2data,ADDPRIV_vptr3data,faceptr);
    }
}


void PR_LightFunc_LIGHTMAPA (void)    /* 17 */
{
PR_FACE_DATA_REAL *faceptr;

  if (ADDPRIV_setup_state)
    {
     if (last_material != ADDPRIV_mat)
       {
        PR_SelectTexture (ADDPRIV_mat->texture_number);
        last_material = ADDPRIV_mat;
       }

     if (last_method != 69)
       {
        guColorCombineFunction (GR_COLORCOMBINE_DECAL_TEXTURE);
        grTexCombineFunction (PR_CurrentTMU, GR_TEXTURECOMBINE_DECAL);
        grDepthBiasLevel (-1);

        guAlphaSource (GR_ALPHASOURCE_ITERATED_ALPHA);
        grAlphaBlendFunction (GR_BLEND_SRC_ALPHA,
                GR_BLEND_ONE, GR_BLEND_ONE, GR_BLEND_ZERO);
        grAlphaTestFunction(GR_CMP_ALWAYS);

        last_method = 69;
       }
    }


  if (ADDPRIV_setup_data)
    {
     ADDPRIV_vptr1data->a = ADDPRIV_mat->a;
     ADDPRIV_vptr2data->a = ADDPRIV_mat->a;
     ADDPRIV_vptr3data->a = ADDPRIV_mat->a;

     faceptr = (PR_FACE_DATA_REAL *)&ADDPRIV_face->face_data;
     SET_TMAP(ADDPRIV_mat, ADDPRIV_vptr1data, ADDPRIV_vptr2data, ADDPRIV_vptr3data, ADDPRIV_vptr1, ADDPRIV_vptr2, ADDPRIV_vptr3, ADDPRIV_vptr1data,ADDPRIV_vptr2data,ADDPRIV_vptr3data,faceptr);
    }
}


void PR_LightFunc_LIGHTMAPM (void)    /* 18 */
{
PR_FACE_DATA_REAL *faceptr;

  if (ADDPRIV_setup_state)
    {
     if (last_material != ADDPRIV_mat)
       {
        PR_SelectTexture (ADDPRIV_mat->texture_number);
        last_material = ADDPRIV_mat;
       }

     if (last_method != 70)
       {
        grDepthBiasLevel (-1);
        grColorCombine (GR_COMBINE_FUNCTION_SCALE_OTHER, GR_COMBINE_FACTOR_LOCAL,
                        GR_COMBINE_LOCAL_ITERATED, GR_COMBINE_OTHER_TEXTURE, FXFALSE);
        grAlphaCombine (GR_COMBINE_FUNCTION_SCALE_OTHER, GR_COMBINE_FACTOR_LOCAL,
                        GR_COMBINE_LOCAL_ITERATED, GR_COMBINE_OTHER_TEXTURE, FXFALSE);
        grTexCombine (PR_CurrentTMU, GR_COMBINE_FUNCTION_LOCAL, GR_COMBINE_FACTOR_NONE,
		GR_COMBINE_FUNCTION_LOCAL, GR_COMBINE_FACTOR_NONE, FXFALSE, FXFALSE);
        grAlphaBlendFunction (GR_BLEND_DST_COLOR, GR_BLEND_ZERO, 
                              GR_BLEND_ONE, GR_BLEND_ZERO);

        guColorCombineFunction (GR_COLORCOMBINE_DECAL_TEXTURE);
        grAlphaTestFunction(GR_CMP_ALWAYS);

        last_method = 70;
       }
    }


  if (ADDPRIV_setup_data)
    {
     ADDPRIV_vptr1data->a = ADDPRIV_mat->a;
     ADDPRIV_vptr2data->a = ADDPRIV_mat->a;
     ADDPRIV_vptr3data->a = ADDPRIV_mat->a;

     faceptr = (PR_FACE_DATA_REAL *)&ADDPRIV_face->face_data;
     SET_TMAP(ADDPRIV_mat, ADDPRIV_vptr1data, ADDPRIV_vptr2data, ADDPRIV_vptr3data, ADDPRIV_vptr1, ADDPRIV_vptr2, ADDPRIV_vptr3, ADDPRIV_vptr1data,ADDPRIV_vptr2data,ADDPRIV_vptr3data,faceptr);
    }
}


void PR_LightFunc_TRANSGOUR (void)
{
PR_REAL i;


  if (ADDPRIV_setup_state)
    {
     if (last_method != 71)
       {
        guColorCombineFunction (GR_COLORCOMBINE_ITRGB);

        grDepthBiasLevel (0);
        guAlphaSource( GR_ALPHASOURCE_ITERATED_ALPHA);
        grAlphaBlendFunction (GR_BLEND_SRC_ALPHA, GR_BLEND_ONE_MINUS_SRC_ALPHA, GR_BLEND_ONE, GR_BLEND_ZERO);
        grAlphaTestFunction(GR_CMP_ALWAYS);
        last_method = 71;
       }
    }


  if (ADDPRIV_setup_data)
    {
     PR_WORKFACEFLAGS = !(ADDPRIV_face->flags & FFLAG_BACK_VISIBLE);

     if (ADDPRIV_vptr1data->dotproduct != ADDPRIV_material_num)
       {
        ADDPRIV_vptr1data->dotproduct = ADDPRIV_material_num;
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr1, ADDPRIV_vptr1data);

        i = ADDPRIV_facedatareal->col[0];          /* Add the precalced light */
        if (i > 0)
          {
           ADDPRIV_r += ADDPRIV_mat->r * i;
           ADDPRIV_g += ADDPRIV_mat->g * i;
           ADDPRIV_b += ADDPRIV_mat->b * i;
           CLAMP_RGB;
          }

        ADDPRIV_vptr1data->r = ADDPRIV_r;
        ADDPRIV_vptr1data->g = ADDPRIV_g;
        ADDPRIV_vptr1data->b = ADDPRIV_b;
        ADDPRIV_vptr1data->a = ADDPRIV_mat->a;
       }

     if (ADDPRIV_vptr2data->dotproduct != ADDPRIV_material_num)
       {
        ADDPRIV_vptr2data->dotproduct = ADDPRIV_material_num;
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr2, ADDPRIV_vptr2data);

        i = ADDPRIV_facedatareal->col[1];          /* Add the precalced light */
        if (i > 0)
          {
           ADDPRIV_r += ADDPRIV_mat->r * i;
           ADDPRIV_g += ADDPRIV_mat->g * i;
           ADDPRIV_b += ADDPRIV_mat->b * i;
           CLAMP_RGB;
          }

        ADDPRIV_vptr2data->r = ADDPRIV_r;
        ADDPRIV_vptr2data->g = ADDPRIV_g;
        ADDPRIV_vptr2data->b = ADDPRIV_b;
        ADDPRIV_vptr2data->a = ADDPRIV_mat->a;
       }

     if (ADDPRIV_vptr3data->dotproduct != ADDPRIV_material_num)
       {
        ADDPRIV_vptr3data->dotproduct = ADDPRIV_material_num;
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr3, ADDPRIV_vptr3data);

        i = ADDPRIV_facedatareal->col[2];          /* Add the precalced light */
        if (i > 0)
          {
           ADDPRIV_r += ADDPRIV_mat->r * i;
           ADDPRIV_g += ADDPRIV_mat->g * i;
           ADDPRIV_b += ADDPRIV_mat->b * i;
           CLAMP_RGB;
          }

        ADDPRIV_vptr3data->r = ADDPRIV_r;
        ADDPRIV_vptr3data->g = ADDPRIV_g;
        ADDPRIV_vptr3data->b = ADDPRIV_b;
        ADDPRIV_vptr3data->a = ADDPRIV_mat->a;
       }
    }
}


void PR_LightFunc_TRANSGOURTEX (void)
{
PR_REAL i;
PR_FACE_DATA_REAL *faceptr;

  if (ADDPRIV_setup_state)
    {
     if (last_material != ADDPRIV_mat)
       {
        PR_SelectTexture (ADDPRIV_mat->texture_number);
        last_material = ADDPRIV_mat;
       }

     if (last_method != 72)
       {
        guColorCombineFunction (GR_COLORCOMBINE_TEXTURE_TIMES_ITRGB);
        grTexCombineFunction (PR_CurrentTMU, GR_TEXTURECOMBINE_DECAL);

        grDepthBiasLevel (0);
        guAlphaSource( GR_ALPHASOURCE_ITERATED_ALPHA);
        grAlphaBlendFunction (GR_BLEND_SRC_ALPHA, GR_BLEND_ONE_MINUS_SRC_ALPHA, GR_BLEND_ONE, GR_BLEND_ZERO);
        grAlphaTestFunction(GR_CMP_ALWAYS);
        last_method = 72;
       }
    }


  if (ADDPRIV_setup_data)
    {
     PR_WORKFACEFLAGS = !(ADDPRIV_face->flags & FFLAG_BACK_VISIBLE);

     if (ADDPRIV_vptr1data->dotproduct != ADDPRIV_material_num)
       {
        ADDPRIV_vptr1data->dotproduct = ADDPRIV_material_num;
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr1, ADDPRIV_vptr1data);

        i = ADDPRIV_facedatareal->col[0];          /* Add the precalced light */
        if (i > 0)
          {
           ADDPRIV_r += ADDPRIV_mat->r * i;
           ADDPRIV_g += ADDPRIV_mat->g * i;
           ADDPRIV_b += ADDPRIV_mat->b * i;
           CLAMP_RGB;
          }

        ADDPRIV_vptr1data->r = ADDPRIV_r;
        ADDPRIV_vptr1data->g = ADDPRIV_g;
        ADDPRIV_vptr1data->b = ADDPRIV_b;
        ADDPRIV_vptr1data->a = ADDPRIV_mat->a;
       }

     if (ADDPRIV_vptr2data->dotproduct != ADDPRIV_material_num)
       {
        ADDPRIV_vptr2data->dotproduct = ADDPRIV_material_num;
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr2, ADDPRIV_vptr2data);

        i = ADDPRIV_facedatareal->col[1];          /* Add the precalced light */
        if (i > 0)
          {
           ADDPRIV_r += ADDPRIV_mat->r * i;
           ADDPRIV_g += ADDPRIV_mat->g * i;
           ADDPRIV_b += ADDPRIV_mat->b * i;
           CLAMP_RGB;
          }

        ADDPRIV_vptr2data->r = ADDPRIV_r;
        ADDPRIV_vptr2data->g = ADDPRIV_g;
        ADDPRIV_vptr2data->b = ADDPRIV_b;
        ADDPRIV_vptr2data->a = ADDPRIV_mat->a;
       }

     if (ADDPRIV_vptr3data->dotproduct != ADDPRIV_material_num)
       {
        ADDPRIV_vptr3data->dotproduct = ADDPRIV_material_num;
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr3, ADDPRIV_vptr3data);

        i = ADDPRIV_facedatareal->col[2];          /* Add the precalced light */
        if (i > 0)
          {
           ADDPRIV_r += ADDPRIV_mat->r * i;
           ADDPRIV_g += ADDPRIV_mat->g * i;
           ADDPRIV_b += ADDPRIV_mat->b * i;
           CLAMP_RGB;
          }

        ADDPRIV_vptr3data->r = ADDPRIV_r;
        ADDPRIV_vptr3data->g = ADDPRIV_g;
        ADDPRIV_vptr3data->b = ADDPRIV_b;
        ADDPRIV_vptr3data->a = ADDPRIV_mat->a;
       }

     faceptr = (PR_FACE_DATA_REAL *)&ADDPRIV_face->face_data;
     SET_TMAP(ADDPRIV_mat, ADDPRIV_vptr1data, ADDPRIV_vptr2data, ADDPRIV_vptr3data, ADDPRIV_vptr1, ADDPRIV_vptr2, ADDPRIV_vptr3, ADDPRIV_vptr1data,ADDPRIV_vptr2data,ADDPRIV_vptr3data,faceptr);
    }
}



void PR_LightFunc_RGBGOUR (void) 
{
PR_DWORD packedrgb;

  if (ADDPRIV_setup_state)
    {
     if (last_method != 73)
       {
        guColorCombineFunction (GR_COLORCOMBINE_ITRGB);

        grDepthBiasLevel (0);
        grAlphaTestFunction (GR_CMP_ALWAYS);
        grAlphaBlendFunction (GR_BLEND_ONE, GR_BLEND_ZERO,
                              GR_BLEND_ONE, GR_BLEND_ZERO);
        last_method = 73;
       }
    }


  if (ADDPRIV_setup_data)
    {
     packedrgb = *(PR_DWORD *)&ADDPRIV_facedatareal->col[0];
     ADDPRIV_vptr1data->r = (packedrgb)       & 0xFF;
     ADDPRIV_vptr1data->g = (packedrgb >> 8)  & 0xFF;
     ADDPRIV_vptr1data->b = (packedrgb >> 16) & 0xFF;
     ADDPRIV_vptr1data->a = ADDPRIV_mat->a;

     packedrgb = *(PR_DWORD *)&ADDPRIV_facedatareal->col[1];
     ADDPRIV_vptr2data->r = (packedrgb)       & 0xFF;
     ADDPRIV_vptr2data->g = (packedrgb >> 8)  & 0xFF;
     ADDPRIV_vptr2data->b = (packedrgb >> 16) & 0xFF;
     ADDPRIV_vptr2data->a = ADDPRIV_mat->a;

     packedrgb = *(PR_DWORD *)&ADDPRIV_facedatareal->col[2];
     ADDPRIV_vptr3data->r = (packedrgb)       & 0xFF;
     ADDPRIV_vptr3data->g = (packedrgb >> 8)  & 0xFF;
     ADDPRIV_vptr3data->b = (packedrgb >> 16) & 0xFF;
     ADDPRIV_vptr3data->a = ADDPRIV_mat->a;
    }
}



void PR_LightFunc_RGBTEX (void)
{
PR_FACE_DATA_REAL *faceptr;
PR_DWORD packedrgb;

  if (ADDPRIV_setup_state)
    {
     if (last_material != ADDPRIV_mat)
       {
        PR_SelectTexture (ADDPRIV_mat->texture_number);
        last_material = ADDPRIV_mat;
       }

     if (last_method != 74)
       {
        guColorCombineFunction (GR_COLORCOMBINE_TEXTURE_TIMES_ITRGB);
        grTexCombineFunction (GR_TMU0, GR_TEXTURECOMBINE_DECAL);

        grDepthBiasLevel (0);
        grAlphaBlendFunction (GR_BLEND_ONE, GR_BLEND_ZERO,
                              GR_BLEND_ONE, GR_BLEND_ZERO);
        grAlphaTestFunction (GR_CMP_ALWAYS);
        last_method = 74;
       }
    }


  if (ADDPRIV_setup_data)
    {
     PR_WORKFACEFLAGS = !(ADDPRIV_face->flags & FFLAG_BACK_VISIBLE);

     packedrgb = *(PR_DWORD *)&ADDPRIV_facedatareal->col[0];
     ADDPRIV_vptr1data->r = (packedrgb)       & 0xFF;
     ADDPRIV_vptr1data->g = (packedrgb >> 8)  & 0xFF;
     ADDPRIV_vptr1data->b = (packedrgb >> 16) & 0xFF;
     ADDPRIV_vptr1data->a = ADDPRIV_mat->a;

     packedrgb = *(PR_DWORD *)&ADDPRIV_facedatareal->col[1];
     ADDPRIV_vptr2data->r = (packedrgb)       & 0xFF;
     ADDPRIV_vptr2data->g = (packedrgb >> 8)  & 0xFF;
     ADDPRIV_vptr2data->b = (packedrgb >> 16) & 0xFF;
     ADDPRIV_vptr2data->a = ADDPRIV_mat->a;

     packedrgb = *(PR_DWORD *)&ADDPRIV_facedatareal->col[2];
     ADDPRIV_vptr3data->r = (packedrgb)       & 0xFF;
     ADDPRIV_vptr3data->g = (packedrgb >> 8)  & 0xFF;
     ADDPRIV_vptr3data->b = (packedrgb >> 16) & 0xFF;
     ADDPRIV_vptr3data->a = ADDPRIV_mat->a;

     faceptr = (PR_FACE_DATA_REAL *)&ADDPRIV_face->face_data;
     SET_TMAP(ADDPRIV_mat, ADDPRIV_vptr1data, ADDPRIV_vptr2data, ADDPRIV_vptr3data, ADDPRIV_vptr1, ADDPRIV_vptr2, ADDPRIV_vptr3, ADDPRIV_vptr1data,ADDPRIV_vptr2data,ADDPRIV_vptr3data,faceptr);
    }
}



void PR_LightFunc_RGBGOUR_DYN (void) 
{
PR_DWORD packedrgb;

  if (ADDPRIV_setup_state)
    {
     if (last_method != 75)
       {
        guColorCombineFunction (GR_COLORCOMBINE_ITRGB);

        grDepthBiasLevel (0);
        grAlphaTestFunction (GR_CMP_ALWAYS);
        grAlphaBlendFunction (GR_BLEND_ONE, GR_BLEND_ZERO,
                              GR_BLEND_ONE, GR_BLEND_ZERO);
        last_method = 75;
       }
    }


  if (ADDPRIV_setup_data)
    {
     packedrgb = *(PR_DWORD *)&ADDPRIV_facedatareal->col[0];
     ADDPRIV_vptr1data->r = (packedrgb)       & 0xFF;
     ADDPRIV_vptr1data->g = (packedrgb >> 8)  & 0xFF;
     ADDPRIV_vptr1data->b = (packedrgb >> 16) & 0xFF;
     ADDPRIV_vptr1data->a = ADDPRIV_mat->a;

     packedrgb = *(PR_DWORD *)&ADDPRIV_facedatareal->col[1];
     ADDPRIV_vptr2data->r = (packedrgb)       & 0xFF;
     ADDPRIV_vptr2data->g = (packedrgb >> 8)  & 0xFF;
     ADDPRIV_vptr2data->b = (packedrgb >> 16) & 0xFF;
     ADDPRIV_vptr2data->a = ADDPRIV_mat->a;

     packedrgb = *(PR_DWORD *)&ADDPRIV_facedatareal->col[2];
     ADDPRIV_vptr3data->r = (packedrgb)       & 0xFF;
     ADDPRIV_vptr3data->g = (packedrgb >> 8)  & 0xFF;
     ADDPRIV_vptr3data->b = (packedrgb >> 16) & 0xFF;
     ADDPRIV_vptr3data->a = ADDPRIV_mat->a;


     ADDPRIV_vptr1data->dotproduct = -1;
     PR_CalcVertexLighting3Dfx (ADDPRIV_vptr1, ADDPRIV_vptr1data);

     ADDPRIV_vptr1data->r += ADDPRIV_r;
     ADDPRIV_vptr1data->g += ADDPRIV_g;
     ADDPRIV_vptr1data->b += ADDPRIV_b;

     if (ADDPRIV_vptr1data->r > 255)
       ADDPRIV_vptr1data->r = 255;
     if (ADDPRIV_vptr1data->g > 255)
       ADDPRIV_vptr1data->g = 255;
     if (ADDPRIV_vptr1data->b > 255)
       ADDPRIV_vptr1data->b = 255;


     ADDPRIV_vptr2data->dotproduct = -1;
     PR_CalcVertexLighting3Dfx (ADDPRIV_vptr2, ADDPRIV_vptr2data);

     ADDPRIV_vptr2data->r += ADDPRIV_r;
     ADDPRIV_vptr2data->g += ADDPRIV_g;
     ADDPRIV_vptr2data->b += ADDPRIV_b;

     if (ADDPRIV_vptr2data->r > 255)
       ADDPRIV_vptr2data->r = 255;
     if (ADDPRIV_vptr2data->g > 255)
       ADDPRIV_vptr2data->g = 255;
     if (ADDPRIV_vptr2data->b > 255)
       ADDPRIV_vptr2data->b = 255;


     ADDPRIV_vptr3data->dotproduct = -1;
     PR_CalcVertexLighting3Dfx (ADDPRIV_vptr3, ADDPRIV_vptr3data);

     ADDPRIV_vptr3data->r += ADDPRIV_r;
     ADDPRIV_vptr3data->g += ADDPRIV_g;
     ADDPRIV_vptr3data->b += ADDPRIV_b;

     if (ADDPRIV_vptr3data->r > 255)
       ADDPRIV_vptr3data->r = 255;
     if (ADDPRIV_vptr3data->g > 255)
       ADDPRIV_vptr3data->g = 255;
     if (ADDPRIV_vptr3data->b > 255)
       ADDPRIV_vptr3data->b = 255;
    }
}



void PR_LightFunc_RGBTEX_DYN (void)
{
PR_FACE_DATA_REAL *faceptr;
PR_DWORD packedrgb;

  if (ADDPRIV_setup_state)
    {
     if (last_material != ADDPRIV_mat)
       {
        PR_SelectTexture (ADDPRIV_mat->texture_number);
        last_material = ADDPRIV_mat;
       }

     if (last_method != 76)
       {
        guColorCombineFunction (GR_COLORCOMBINE_TEXTURE_TIMES_ITRGB);
        grTexCombineFunction (GR_TMU0, GR_TEXTURECOMBINE_DECAL);

        grDepthBiasLevel (0);
        grAlphaBlendFunction (GR_BLEND_ONE, GR_BLEND_ZERO,
                              GR_BLEND_ONE, GR_BLEND_ZERO);
        grAlphaTestFunction (GR_CMP_ALWAYS);
        last_method = 76;
       }
    }


  if (ADDPRIV_setup_data)
    {
     PR_WORKFACEFLAGS = !(ADDPRIV_face->flags & FFLAG_BACK_VISIBLE);

     packedrgb = *(PR_DWORD *)&ADDPRIV_facedatareal->col[0];
     ADDPRIV_vptr1data->r = (packedrgb)       & 0xFF;
     ADDPRIV_vptr1data->g = (packedrgb >> 8)  & 0xFF;
     ADDPRIV_vptr1data->b = (packedrgb >> 16) & 0xFF;
     ADDPRIV_vptr1data->a = ADDPRIV_mat->a;

     packedrgb = *(PR_DWORD *)&ADDPRIV_facedatareal->col[1];
     ADDPRIV_vptr2data->r = (packedrgb)       & 0xFF;
     ADDPRIV_vptr2data->g = (packedrgb >> 8)  & 0xFF;
     ADDPRIV_vptr2data->b = (packedrgb >> 16) & 0xFF;
     ADDPRIV_vptr2data->a = ADDPRIV_mat->a;

     packedrgb = *(PR_DWORD *)&ADDPRIV_facedatareal->col[2];
     ADDPRIV_vptr3data->r = (packedrgb)       & 0xFF;
     ADDPRIV_vptr3data->g = (packedrgb >> 8)  & 0xFF;
     ADDPRIV_vptr3data->b = (packedrgb >> 16) & 0xFF;
     ADDPRIV_vptr3data->a = ADDPRIV_mat->a;

       {
        ADDPRIV_vptr1data->dotproduct = -1;
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr1, ADDPRIV_vptr1data);

        ADDPRIV_vptr1data->r += ADDPRIV_r;
        ADDPRIV_vptr1data->g += ADDPRIV_g;
        ADDPRIV_vptr1data->b += ADDPRIV_b;

        if (ADDPRIV_vptr1data->r > 255)
          ADDPRIV_vptr1data->r = 255;
        if (ADDPRIV_vptr1data->g > 255)
          ADDPRIV_vptr1data->g = 255;
        if (ADDPRIV_vptr1data->b > 255)
          ADDPRIV_vptr1data->b = 255;
       }

       {
        ADDPRIV_vptr2data->dotproduct = -1;
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr2, ADDPRIV_vptr2data);

        ADDPRIV_vptr2data->r += ADDPRIV_r;
        ADDPRIV_vptr2data->g += ADDPRIV_g;
        ADDPRIV_vptr2data->b += ADDPRIV_b;

        if (ADDPRIV_vptr2data->r > 255)
          ADDPRIV_vptr2data->r = 255;
        if (ADDPRIV_vptr2data->g > 255)
          ADDPRIV_vptr2data->g = 255;
        if (ADDPRIV_vptr2data->b > 255)
          ADDPRIV_vptr2data->b = 255;
       }

       {
        ADDPRIV_vptr3data->dotproduct = -1;
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr3, ADDPRIV_vptr3data);

        ADDPRIV_vptr3data->r += ADDPRIV_r;
        ADDPRIV_vptr3data->g += ADDPRIV_g;
        ADDPRIV_vptr3data->b += ADDPRIV_b;

        if (ADDPRIV_vptr3data->r > 255)
          ADDPRIV_vptr3data->r = 255;
        if (ADDPRIV_vptr3data->g > 255)
          ADDPRIV_vptr3data->g = 255;
        if (ADDPRIV_vptr3data->b > 255)
          ADDPRIV_vptr3data->b = 255;
       }


     faceptr = (PR_FACE_DATA_REAL *)&ADDPRIV_face->face_data;
     SET_TMAP(ADDPRIV_mat, ADDPRIV_vptr1data, ADDPRIV_vptr2data, ADDPRIV_vptr3data, ADDPRIV_vptr1, ADDPRIV_vptr2, ADDPRIV_vptr3, ADDPRIV_vptr1data,ADDPRIV_vptr2data,ADDPRIV_vptr3data,faceptr);
    }
}



void PR_LightFunc_FOG_GOURAUD (void)  
{
PR_DWORD fogged, fogres;

 
  if (ADDPRIV_setup_data)
    {
     PR_WORKFACEFLAGS = !(ADDPRIV_face->flags & FFLAG_BACK_VISIBLE);
     if (ADDPRIV_vptr1data->dotproduct != ADDPRIV_material_num)
       {
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr1, ADDPRIV_vptr1data);

        ADDPRIV_vptr1data->dotproduct = ADDPRIV_material_num;
   
        ADDPRIV_vptr1data->r = ADDPRIV_r;
        ADDPRIV_vptr1data->g = ADDPRIV_g;
        ADDPRIV_vptr1data->b = ADDPRIV_b;
        ADDPRIV_vptr1data->a = 255.0f;

        fogres = AddAlphaFog1 ();
        if (fogres)
          ADDPRIV_vptr1data->dotproduct = -1;
       }

     if (ADDPRIV_vptr2data->dotproduct != ADDPRIV_material_num)
       {
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr2, ADDPRIV_vptr2data);
   
        ADDPRIV_vptr2data->dotproduct = ADDPRIV_material_num;
   
        ADDPRIV_vptr2data->r = ADDPRIV_r;
        ADDPRIV_vptr2data->g = ADDPRIV_g;
        ADDPRIV_vptr2data->b = ADDPRIV_b;
        ADDPRIV_vptr2data->a = 255.0f;

        fogres = AddAlphaFog2 ();
        if (fogres)
          ADDPRIV_vptr2data->dotproduct = -1;
       }

     if (ADDPRIV_vptr3data->dotproduct != ADDPRIV_material_num)
       {
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr3, ADDPRIV_vptr3data);
   
        ADDPRIV_vptr3data->dotproduct = ADDPRIV_material_num;
   
        ADDPRIV_vptr3data->r = ADDPRIV_r;
        ADDPRIV_vptr3data->g = ADDPRIV_g;
        ADDPRIV_vptr3data->b = ADDPRIV_b;
        ADDPRIV_vptr3data->a = 255.0f;

        fogres = AddAlphaFog3 ();
        if (fogres)
          ADDPRIV_vptr3data->dotproduct = -1;
       }
    }


  if (ADDPRIV_setup_state)
    {
     if (last_method != 77)
       {
        guColorCombineFunction (GR_COLORCOMBINE_ITRGB);
        guAlphaSource (GR_ALPHASOURCE_ITERATED_ALPHA);
        grAlphaTestFunction (GR_CMP_ALWAYS);
        grDepthBiasLevel (0);

        last_method = 77;
        last_fog = -1;
       }

     fogged = ((ADDPRIV_vptr1data->dotproduct == -1) ||
               (ADDPRIV_vptr2data->dotproduct == -1) ||
               (ADDPRIV_vptr3data->dotproduct == -1));
     if (fogged != last_fog)
       {
        if (fogged)
          grAlphaBlendFunction (GR_BLEND_SRC_ALPHA, GR_BLEND_ONE_MINUS_SRC_ALPHA, GR_BLEND_ONE, GR_BLEND_ZERO);
        else
          grAlphaBlendFunction (GR_BLEND_ONE, GR_BLEND_ZERO,
                                GR_BLEND_ONE, GR_BLEND_ZERO);
        last_fog = fogged;
       }

    }
}



void PR_LightFunc_FOG_DECAL (void)    
{
PR_FACE_DATA_REAL *faceptr;
PR_DWORD fogres, fogged;

  if (ADDPRIV_setup_data)
    {
     if (ADDPRIV_vptr1data->dotproduct != ADDPRIV_material_num)
       {
        ADDPRIV_vptr1data->dotproduct = ADDPRIV_material_num;
        ADDPRIV_vptr1data->a = 255.0f;

        fogres = AddAlphaFog1 ();
        if (fogres)
          ADDPRIV_vptr1data->dotproduct = -1;
       }

     if (ADDPRIV_vptr2data->dotproduct != ADDPRIV_material_num)
       {
        ADDPRIV_vptr2data->dotproduct = ADDPRIV_material_num;
        ADDPRIV_vptr2data->a = 255.0f;

        fogres = AddAlphaFog2 ();
        if (fogres)
          ADDPRIV_vptr2data->dotproduct = -1;
       }

     if (ADDPRIV_vptr3data->dotproduct != ADDPRIV_material_num)
       {
        ADDPRIV_vptr3data->dotproduct = ADDPRIV_material_num;
        ADDPRIV_vptr3data->a = 255.0f;

        fogres = AddAlphaFog3 ();
        if (fogres)
          ADDPRIV_vptr3data->dotproduct = -1;
       }

     faceptr = (PR_FACE_DATA_REAL *)&ADDPRIV_face->face_data;
     SET_TMAP(ADDPRIV_mat, ADDPRIV_vptr1data, ADDPRIV_vptr2data, ADDPRIV_vptr3data, ADDPRIV_vptr1, ADDPRIV_vptr2, ADDPRIV_vptr3, ADDPRIV_vptr1data,ADDPRIV_vptr2data,ADDPRIV_vptr3data,faceptr);
    }


  if (ADDPRIV_setup_state)
    {
     if (last_material != ADDPRIV_mat)
       {
        PR_SelectTexture (ADDPRIV_mat->texture_number);
        last_material = ADDPRIV_mat;
       }

     if (last_method != 78)
       {
        guColorCombineFunction (GR_COLORCOMBINE_DECAL_TEXTURE);
        grTexCombineFunction (PR_CurrentTMU, GR_TEXTURECOMBINE_DECAL);

        grDepthBiasLevel (0);

        grAlphaTestFunction (GR_CMP_ALWAYS);
        grAlphaBlendFunction (GR_BLEND_ONE, GR_BLEND_ZERO,
                              GR_BLEND_ONE, GR_BLEND_ZERO);
        last_method = 78;
        last_fog = -1;
       }


     fogged = ((ADDPRIV_vptr1data->dotproduct == -1) ||
               (ADDPRIV_vptr2data->dotproduct == -1) ||
               (ADDPRIV_vptr3data->dotproduct == -1));
     if (fogged != last_fog)
       {
        if (fogged)
          {
           guAlphaSource(GR_ALPHASOURCE_ITERATED_ALPHA);
           grAlphaBlendFunction (GR_BLEND_SRC_ALPHA, GR_BLEND_ONE_MINUS_SRC_ALPHA, GR_BLEND_ONE, GR_BLEND_ZERO);
          }
        else
          grAlphaBlendFunction (GR_BLEND_ONE, GR_BLEND_ZERO,
                                GR_BLEND_ONE, GR_BLEND_ZERO);
        last_fog = fogged;
       }

    }
}


void PR_LightFunc_FOG_XDECAL (void)    
{
PR_FACE_DATA_REAL *faceptr;
PR_DWORD fogres;

  if (ADDPRIV_setup_data)
    {
     if (ADDPRIV_vptr1data->dotproduct != ADDPRIV_material_num)
       {
        ADDPRIV_vptr1data->dotproduct = ADDPRIV_material_num;
        ADDPRIV_vptr1data->a = 255.0f;

        fogres = AddAlphaFog1 ();
        if (fogres)
          ADDPRIV_vptr1data->dotproduct = -1;
       }

     if (ADDPRIV_vptr2data->dotproduct != ADDPRIV_material_num)
       {
        ADDPRIV_vptr2data->dotproduct = ADDPRIV_material_num;
        ADDPRIV_vptr2data->a = 255.0f;

        fogres = AddAlphaFog2 ();
        if (fogres)
          ADDPRIV_vptr2data->dotproduct = -1;
       }

     if (ADDPRIV_vptr3data->dotproduct != ADDPRIV_material_num)
       {
        ADDPRIV_vptr3data->dotproduct = ADDPRIV_material_num;
        ADDPRIV_vptr3data->a = 255.0f;

        fogres = AddAlphaFog3 ();
        if (fogres)
          ADDPRIV_vptr3data->dotproduct = -1;
       }

     faceptr = (PR_FACE_DATA_REAL *)&ADDPRIV_face->face_data;
     SET_TMAP(ADDPRIV_mat, ADDPRIV_vptr1data, ADDPRIV_vptr2data, ADDPRIV_vptr3data, ADDPRIV_vptr1, ADDPRIV_vptr2, ADDPRIV_vptr3, ADDPRIV_vptr1data,ADDPRIV_vptr2data,ADDPRIV_vptr3data,faceptr);
    }


  if (ADDPRIV_setup_state)
    {
     if (last_material != ADDPRIV_mat)
       {
        PR_SelectTexture (ADDPRIV_mat->texture_number);
        last_material = ADDPRIV_mat;
       }

     if (last_method != 79)
       {
        guColorCombineFunction (GR_COLORCOMBINE_DECAL_TEXTURE);
        grTexCombineFunction (PR_CurrentTMU, GR_TEXTURECOMBINE_DECAL);

        grDepthBiasLevel (0);
        grAlphaTestReferenceValue (1);
        grAlphaTestFunction (GR_CMP_GREATER);

        grAlphaCombine(GR_COMBINE_FUNCTION_BLEND,
                       GR_COMBINE_FACTOR_ONE,
                       GR_COMBINE_LOCAL_NONE,
                       GR_COMBINE_OTHER_TEXTURE,
                       FXFALSE );
        guAlphaSource (GR_ALPHASOURCE_TEXTURE_ALPHA_TIMES_ITERATED_ALPHA);
        grAlphaBlendFunction (GR_BLEND_SRC_ALPHA, GR_BLEND_ONE_MINUS_SRC_ALPHA,
                              GR_BLEND_ONE, GR_BLEND_ZERO);
        last_method = 79;
       }
    }
}




void PR_LightFunc_FOG_FLAT_TEXTURED (void)  
{
PR_FACE_DATA_REAL *faceptr;
PR_REAL i;
PR_DWORD fogres, fogged;

  if (ADDPRIV_setup_data)
    {
     PR_WORKFACEFLAGS = !(ADDPRIV_face->flags & FFLAG_BACK_VISIBLE);
     PR_CalcFaceLighting3Dfx (ADDPRIV_face);

     i = ADDPRIV_facedatareal->col[0];          /* Add the precalced light */
     if (i > 0)
       {
        ADDPRIV_r += ADDPRIV_mat->r * i;
        ADDPRIV_g += ADDPRIV_mat->g * i;
        ADDPRIV_b += ADDPRIV_mat->b * i;
        CLAMP_RGB;
       }

     ADDPRIV_vptr1data->r = ADDPRIV_vptr2data->r = ADDPRIV_vptr3data->r = ADDPRIV_r;
     ADDPRIV_vptr1data->a = 255.0f;
     ADDPRIV_vptr1data->g = ADDPRIV_vptr2data->g = ADDPRIV_vptr3data->g = ADDPRIV_g;
     ADDPRIV_vptr2data->a = 255.0f;
     ADDPRIV_vptr1data->b = ADDPRIV_vptr2data->b = ADDPRIV_vptr3data->b = ADDPRIV_b;
     ADDPRIV_vptr3data->a = 255.0f;

     fogres = AddAlphaFog1 ();
     if (fogres)
       ADDPRIV_vptr1data->dotproduct = -1;

     fogres = AddAlphaFog2 ();
     if (fogres)
       ADDPRIV_vptr2data->dotproduct = -1;

     fogres = AddAlphaFog3 ();
     if (fogres)
       ADDPRIV_vptr3data->dotproduct = -1;


     faceptr = (PR_FACE_DATA_REAL *)&ADDPRIV_face->face_data;
     SET_TMAP(ADDPRIV_mat, ADDPRIV_vptr1data, ADDPRIV_vptr2data, ADDPRIV_vptr3data, ADDPRIV_vptr1, ADDPRIV_vptr2, ADDPRIV_vptr3, ADDPRIV_vptr1data,ADDPRIV_vptr2data,ADDPRIV_vptr3data,faceptr);
    }


  if (ADDPRIV_setup_state)
    {
     if (last_material != ADDPRIV_mat)
       {
        PR_SelectTexture (ADDPRIV_mat->texture_number);
        last_material = ADDPRIV_mat;
       }

     if (last_method != 80)
       {
        guColorCombineFunction (GR_COLORCOMBINE_TEXTURE_TIMES_ITRGB);
        grTexCombineFunction (PR_CurrentTMU, GR_TEXTURECOMBINE_DECAL);

        grDepthBiasLevel (0);
        grAlphaTestFunction (GR_CMP_ALWAYS);
        last_method = 80;
        last_fog = -1;
       }

     fogged = ((ADDPRIV_vptr1data->dotproduct == -1) ||
               (ADDPRIV_vptr2data->dotproduct == -1) ||
               (ADDPRIV_vptr3data->dotproduct == -1));
     if (fogged != last_fog)
       {
        if (fogged)
          {
           guAlphaSource(GR_ALPHASOURCE_ITERATED_ALPHA);
           grAlphaBlendFunction (GR_BLEND_SRC_ALPHA, GR_BLEND_ONE_MINUS_SRC_ALPHA, GR_BLEND_ONE, GR_BLEND_ZERO);
          }
        else
          grAlphaBlendFunction (GR_BLEND_ONE, GR_BLEND_ZERO,
                                GR_BLEND_ONE, GR_BLEND_ZERO);
        last_fog = fogged;
       }
    }
}


void PR_LightFunc_FOG_XFLAT_TEXTURED (void)
{
PR_FACE_DATA_REAL *faceptr;
PR_REAL i;
PR_DWORD fogres;


  if (ADDPRIV_setup_data)
    {
     PR_WORKFACEFLAGS = !(ADDPRIV_face->flags & FFLAG_BACK_VISIBLE);
     PR_CalcFaceLighting3Dfx (ADDPRIV_face);

     i = ADDPRIV_facedatareal->col[0];          /* Add the precalced light */
     if (i > 0)
       {
        ADDPRIV_r += ADDPRIV_mat->r * i;
        ADDPRIV_g += ADDPRIV_mat->g * i;
        ADDPRIV_b += ADDPRIV_mat->b * i;
        CLAMP_RGB;
       }

     ADDPRIV_vptr1data->r = ADDPRIV_vptr2data->r = ADDPRIV_vptr3data->r = ADDPRIV_r;
     ADDPRIV_vptr1data->a = 255.0f;
     ADDPRIV_vptr1data->g = ADDPRIV_vptr2data->g = ADDPRIV_vptr3data->g = ADDPRIV_g;
     ADDPRIV_vptr2data->a = 255.0f;
     ADDPRIV_vptr1data->b = ADDPRIV_vptr2data->b = ADDPRIV_vptr3data->b = ADDPRIV_b;
     ADDPRIV_vptr3data->a = 255.0f;

     fogres = AddAlphaFog1 ();
     if (fogres)
       ADDPRIV_vptr1data->dotproduct = -1;

     fogres = AddAlphaFog2 ();
     if (fogres)
       ADDPRIV_vptr2data->dotproduct = -1;

     fogres = AddAlphaFog3 ();
     if (fogres)
       ADDPRIV_vptr3data->dotproduct = -1;

     faceptr = (PR_FACE_DATA_REAL *)&ADDPRIV_face->face_data;
     SET_TMAP(ADDPRIV_mat, ADDPRIV_vptr1data, ADDPRIV_vptr2data, ADDPRIV_vptr3data, ADDPRIV_vptr1, ADDPRIV_vptr2, ADDPRIV_vptr3, ADDPRIV_vptr1data,ADDPRIV_vptr2data,ADDPRIV_vptr3data,faceptr);
    }


  if (ADDPRIV_setup_state)
    {
     if (last_material != ADDPRIV_mat)
       {
        PR_SelectTexture (ADDPRIV_mat->texture_number);
        last_material = ADDPRIV_mat;
       }

     if (last_method != 81)
       {
        guColorCombineFunction (GR_COLORCOMBINE_TEXTURE_TIMES_ITRGB);
        grTexCombineFunction (PR_CurrentTMU, GR_TEXTURECOMBINE_DECAL);

        grAlphaTestReferenceValue (1);
        grAlphaTestFunction (GR_CMP_GREATER);

        grDepthBiasLevel (0);
        grAlphaCombine(GR_COMBINE_FUNCTION_BLEND,
                       GR_COMBINE_FACTOR_ONE,
                       GR_COMBINE_LOCAL_NONE,
                       GR_COMBINE_OTHER_TEXTURE,
                       FXFALSE );
        guAlphaSource (GR_ALPHASOURCE_TEXTURE_ALPHA_TIMES_ITERATED_ALPHA);
        grAlphaBlendFunction (GR_BLEND_SRC_ALPHA, GR_BLEND_ONE_MINUS_SRC_ALPHA,
                              GR_BLEND_ONE, GR_BLEND_ZERO);
                           
        last_method = 81;
       }
    }
}



void PR_LightFunc_FOG_GOURAUD_TEXTURED (void)
{
PR_FACE_DATA_REAL *faceptr;
PR_REAL i;
PR_DWORD fogres, fogged;

  if (ADDPRIV_setup_data)
    {
     PR_WORKFACEFLAGS = !(ADDPRIV_face->flags & FFLAG_BACK_VISIBLE);
     if (ADDPRIV_vptr1data->dotproduct != ADDPRIV_material_num)
       {
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr1, ADDPRIV_vptr1data);

        i = ADDPRIV_facedatareal->col[0];          /* Add the precalced light */
        if (i > 0)
          {
           ADDPRIV_r += ADDPRIV_mat->r * i;
           ADDPRIV_g += ADDPRIV_mat->g * i;
           ADDPRIV_b += ADDPRIV_mat->b * i;
           CLAMP_RGB;
           ADDPRIV_vptr1data->dotproduct = -1;
          }
        else
          ADDPRIV_vptr1data->dotproduct = ADDPRIV_material_num;
   
        ADDPRIV_vptr1data->r = ADDPRIV_r;
        ADDPRIV_vptr1data->g = ADDPRIV_g;
        ADDPRIV_vptr1data->b = ADDPRIV_b;
        ADDPRIV_vptr1data->a = 255.0f;
       }

     if (ADDPRIV_vptr2data->dotproduct != ADDPRIV_material_num)
       {
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr2, ADDPRIV_vptr2data);
   
        i = ADDPRIV_facedatareal->col[1];          /* Add the precalced light */
        if (i > 0)
          {
           ADDPRIV_r += ADDPRIV_mat->r * i;
           ADDPRIV_g += ADDPRIV_mat->g * i;
           ADDPRIV_b += ADDPRIV_mat->b * i;
           CLAMP_RGB;
           ADDPRIV_vptr2data->dotproduct = -1;
          }
        else
          ADDPRIV_vptr2data->dotproduct = ADDPRIV_material_num;
   
        ADDPRIV_vptr2data->r = ADDPRIV_r;
        ADDPRIV_vptr2data->g = ADDPRIV_g;
        ADDPRIV_vptr2data->b = ADDPRIV_b;
        ADDPRIV_vptr2data->a = 255.0f;
       }

     if (ADDPRIV_vptr3data->dotproduct != ADDPRIV_material_num)
       {
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr3, ADDPRIV_vptr3data);
   
        i = ADDPRIV_facedatareal->col[2];          /* Add the precalced light */
        if (i > 0)
          {
           ADDPRIV_r += ADDPRIV_mat->r * i;
           ADDPRIV_g += ADDPRIV_mat->g * i;
           ADDPRIV_b += ADDPRIV_mat->b * i;
           CLAMP_RGB;
           ADDPRIV_vptr3data->dotproduct = -1;
          }
        else
          ADDPRIV_vptr3data->dotproduct = ADDPRIV_material_num;
   
        ADDPRIV_vptr3data->r = ADDPRIV_r;
        ADDPRIV_vptr3data->g = ADDPRIV_g;
        ADDPRIV_vptr3data->b = ADDPRIV_b;
        ADDPRIV_vptr3data->a = 255.0f;
       }

     fogres = AddAlphaFog1 ();
     if (fogres)
       ADDPRIV_vptr1data->dotproduct = -1;

     fogres = AddAlphaFog2 ();
     if (fogres)
       ADDPRIV_vptr2data->dotproduct = -1;

     fogres = AddAlphaFog3 ();
     if (fogres)
       ADDPRIV_vptr3data->dotproduct = -1;

     faceptr = (PR_FACE_DATA_REAL *)&ADDPRIV_face->face_data;
     SET_TMAP(ADDPRIV_mat, ADDPRIV_vptr1data, ADDPRIV_vptr2data, ADDPRIV_vptr3data, ADDPRIV_vptr1, ADDPRIV_vptr2, ADDPRIV_vptr3, ADDPRIV_vptr1data,ADDPRIV_vptr2data,ADDPRIV_vptr3data,faceptr);
    }


  if (ADDPRIV_setup_state)
    {
     if (last_material != ADDPRIV_mat)
       {
        PR_SelectTexture (ADDPRIV_mat->texture_number);
        last_material = ADDPRIV_mat;
       }

     if (last_method != 82)
       {
        guColorCombineFunction (GR_COLORCOMBINE_TEXTURE_TIMES_ITRGB);
        grTexCombineFunction (PR_CurrentTMU, GR_TEXTURECOMBINE_DECAL);

        grDepthBiasLevel (0);
        grAlphaTestFunction (GR_CMP_ALWAYS);

        last_method = 82;
        last_fog = -1;
       }

     fogged = ((ADDPRIV_vptr1data->dotproduct == -1) ||
               (ADDPRIV_vptr2data->dotproduct == -1) ||
               (ADDPRIV_vptr3data->dotproduct == -1));
     if (fogged != last_fog)
       {
        if (fogged)
          {
           guAlphaSource(GR_ALPHASOURCE_ITERATED_ALPHA);
           grAlphaBlendFunction (GR_BLEND_SRC_ALPHA, GR_BLEND_ONE_MINUS_SRC_ALPHA, GR_BLEND_ONE, GR_BLEND_ZERO);
          }
        else
          grAlphaBlendFunction (GR_BLEND_ONE, GR_BLEND_ZERO,
                                GR_BLEND_ONE, GR_BLEND_ZERO);
        last_fog = fogged;
       }
    }
}




void PR_LightFunc_FOG_XGOURAUD_TEXTURED (void)
{
PR_FACE_DATA_REAL *faceptr;
PR_REAL i;
PR_DWORD fogres;


  if (ADDPRIV_setup_data)
    {
     PR_WORKFACEFLAGS = !(ADDPRIV_face->flags & FFLAG_BACK_VISIBLE);
     if (ADDPRIV_vptr1data->dotproduct != ADDPRIV_material_num)
       {
        ADDPRIV_vptr1data->dotproduct = ADDPRIV_material_num;
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr1, ADDPRIV_vptr1data);

        i = ADDPRIV_facedatareal->col[0];          /* Add the precalced light */
        if (i > 0)
          {
           ADDPRIV_r += ADDPRIV_mat->r * i;
           ADDPRIV_g += ADDPRIV_mat->g * i;
           ADDPRIV_b += ADDPRIV_mat->b * i;
           CLAMP_RGB;
          }
        ADDPRIV_vptr1data->r = ADDPRIV_r;
        ADDPRIV_vptr1data->g = ADDPRIV_g;
        ADDPRIV_vptr1data->b = ADDPRIV_b;
        ADDPRIV_vptr1data->a = 255.0f;
       }


     if (ADDPRIV_vptr2data->dotproduct != ADDPRIV_material_num)
       {
        ADDPRIV_vptr2data->dotproduct = ADDPRIV_material_num;
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr2, ADDPRIV_vptr2data);
   
        i = ADDPRIV_facedatareal->col[1];          /* Add the precalced light */
        if (i > 0)
          {
           ADDPRIV_r += ADDPRIV_mat->r * i;
           ADDPRIV_g += ADDPRIV_mat->g * i;
           ADDPRIV_b += ADDPRIV_mat->b * i;
           CLAMP_RGB;
          }
        ADDPRIV_vptr2data->r = ADDPRIV_r;
        ADDPRIV_vptr2data->g = ADDPRIV_g;
        ADDPRIV_vptr2data->b = ADDPRIV_b;
        ADDPRIV_vptr2data->a = 255.0f;
       }

     if (ADDPRIV_vptr3data->dotproduct != ADDPRIV_material_num)
       {
        ADDPRIV_vptr3data->dotproduct = ADDPRIV_material_num;
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr3, ADDPRIV_vptr3data);
   
        i = ADDPRIV_facedatareal->col[2];          /* Add the precalced light */
        if (i > 0)
          {
           ADDPRIV_r += ADDPRIV_mat->r * i;
           ADDPRIV_g += ADDPRIV_mat->g * i;
           ADDPRIV_b += ADDPRIV_mat->b * i;
           CLAMP_RGB;
          }
        ADDPRIV_vptr3data->r = ADDPRIV_r;
        ADDPRIV_vptr3data->g = ADDPRIV_g;
        ADDPRIV_vptr3data->b = ADDPRIV_b;
        ADDPRIV_vptr3data->a = 255.0f;
       }

     fogres = AddAlphaFog1 ();
     if (fogres)
       ADDPRIV_vptr1data->dotproduct = -1;

     fogres = AddAlphaFog2 ();
     if (fogres)
       ADDPRIV_vptr2data->dotproduct = -1;

     fogres = AddAlphaFog3 ();
     if (fogres)
       ADDPRIV_vptr3data->dotproduct = -1;

     faceptr = (PR_FACE_DATA_REAL *)&ADDPRIV_face->face_data;
     SET_TMAP(ADDPRIV_mat, ADDPRIV_vptr1data, ADDPRIV_vptr2data, ADDPRIV_vptr3data, ADDPRIV_vptr1, ADDPRIV_vptr2, ADDPRIV_vptr3, ADDPRIV_vptr1data,ADDPRIV_vptr2data,ADDPRIV_vptr3data,faceptr);
    }


  if (ADDPRIV_setup_state)
    {
     if (last_material != ADDPRIV_mat)
       {
        PR_SelectTexture (ADDPRIV_mat->texture_number);
        last_material = ADDPRIV_mat;
       }

     if (last_method != 83)
       {
        guColorCombineFunction (GR_COLORCOMBINE_TEXTURE_TIMES_ITRGB);
        grTexCombineFunction (PR_CurrentTMU, GR_TEXTURECOMBINE_DECAL);

        grDepthBiasLevel (0);
        grAlphaTestReferenceValue (1);
        grAlphaTestFunction (GR_CMP_GREATER);

        grAlphaCombine(GR_COMBINE_FUNCTION_BLEND,
                       GR_COMBINE_FACTOR_ONE,
                       GR_COMBINE_LOCAL_NONE,
                       GR_COMBINE_OTHER_TEXTURE,
                       FXFALSE );
        guAlphaSource (GR_ALPHASOURCE_TEXTURE_ALPHA_TIMES_ITERATED_ALPHA);
        grAlphaBlendFunction (GR_BLEND_SRC_ALPHA, GR_BLEND_ONE_MINUS_SRC_ALPHA,
                              GR_BLEND_ONE, GR_BLEND_ZERO);

        last_method = 83;
       }
    }


}


void PR_LightFunc_FOG_TRANSLUCENT_TEXTURED (void)
{
PR_FACE_DATA_REAL *faceptr;
PR_DWORD fogres, fogged;

  if (ADDPRIV_setup_data)
    {
     ADDPRIV_vptr1data->a = ADDPRIV_mat->a;
     ADDPRIV_vptr2data->a = ADDPRIV_mat->a;
     ADDPRIV_vptr3data->a = ADDPRIV_mat->a;

     fogres = AddAlphaFog1 ();
     if (fogres)
       ADDPRIV_vptr1data->dotproduct = -1;

     fogres = AddAlphaFog2 ();
     if (fogres)
       ADDPRIV_vptr2data->dotproduct = -1;

     fogres = AddAlphaFog3 ();
     if (fogres)
       ADDPRIV_vptr3data->dotproduct = -1;

     faceptr = (PR_FACE_DATA_REAL *)&ADDPRIV_face->face_data;
     SET_TMAP(ADDPRIV_mat, ADDPRIV_vptr1data, ADDPRIV_vptr2data, ADDPRIV_vptr3data, ADDPRIV_vptr1, ADDPRIV_vptr2, ADDPRIV_vptr3, ADDPRIV_vptr1data,ADDPRIV_vptr2data,ADDPRIV_vptr3data,faceptr);
    }


  if (ADDPRIV_setup_state)
    {
     if (last_material != ADDPRIV_mat)
       {
        PR_SelectTexture (ADDPRIV_mat->texture_number);
        last_material = ADDPRIV_mat;
       }

     if (last_method != 84)
       {
        guColorCombineFunction (GR_COLORCOMBINE_DECAL_TEXTURE);
        grTexCombineFunction (PR_CurrentTMU, GR_TEXTURECOMBINE_DECAL);

        grDepthBiasLevel (0);
        guAlphaSource( GR_ALPHASOURCE_ITERATED_ALPHA);
        grAlphaBlendFunction (GR_BLEND_SRC_ALPHA, GR_BLEND_ONE_MINUS_SRC_ALPHA, GR_BLEND_ONE, GR_BLEND_ZERO);
        grAlphaTestFunction(GR_CMP_ALWAYS);
        last_method = 84;
        last_fog = -1;
       }

     fogged = ((ADDPRIV_vptr1data->dotproduct == -1) ||
               (ADDPRIV_vptr2data->dotproduct == -1) ||
               (ADDPRIV_vptr3data->dotproduct == -1));
     if (fogged != last_fog)
       {
        if (fogged)
          {
           guAlphaSource(GR_ALPHASOURCE_ITERATED_ALPHA);
           grAlphaBlendFunction (GR_BLEND_SRC_ALPHA, GR_BLEND_ONE_MINUS_SRC_ALPHA, GR_BLEND_ONE, GR_BLEND_ZERO);
          }
        else
          grAlphaBlendFunction (GR_BLEND_ONE, GR_BLEND_ZERO,
                                GR_BLEND_ONE, GR_BLEND_ZERO);
        last_fog = fogged;
       }
    }
}


void PR_LightFunc_FOG_XTRANSLUCENT_TEXTURED (void)
{
PR_FACE_DATA_REAL *faceptr;
PR_DWORD fogres;

  if (ADDPRIV_setup_state)
    {
     if (last_material != ADDPRIV_mat)
       {
        PR_SelectTexture (ADDPRIV_mat->texture_number);
        last_material = ADDPRIV_mat;
       }

     if (last_method != 85)
       {
        guColorCombineFunction (GR_COLORCOMBINE_DECAL_TEXTURE);
        grTexCombineFunction (PR_CurrentTMU, GR_TEXTURECOMBINE_DECAL);
   
        grDepthBiasLevel (0);
        grAlphaTestReferenceValue (1);
        grAlphaTestFunction (GR_CMP_GREATER);
   
        grAlphaCombine(GR_COMBINE_FUNCTION_BLEND,
                       GR_COMBINE_FACTOR_ONE,
                       GR_COMBINE_LOCAL_NONE,
                       GR_COMBINE_OTHER_TEXTURE,
                       FXFALSE );
        guAlphaSource (GR_ALPHASOURCE_TEXTURE_ALPHA_TIMES_ITERATED_ALPHA);
        grAlphaBlendFunction (GR_BLEND_SRC_ALPHA, GR_BLEND_ONE_MINUS_SRC_ALPHA, GR_BLEND_ONE, GR_BLEND_ZERO );
        last_method = 85;
       }
    }


  if (ADDPRIV_setup_data)
    {
     ADDPRIV_vptr1data->a = ADDPRIV_mat->a;
     ADDPRIV_vptr2data->a = ADDPRIV_mat->a;
     ADDPRIV_vptr3data->a = ADDPRIV_mat->a;

     fogres = AddAlphaFog1 ();
     if (fogres)
       ADDPRIV_vptr1data->dotproduct = -1;

     fogres = AddAlphaFog2 ();
     if (fogres)
       ADDPRIV_vptr2data->dotproduct = -1;

     fogres = AddAlphaFog3 ();
     if (fogres)
       ADDPRIV_vptr3data->dotproduct = -1;

     faceptr = (PR_FACE_DATA_REAL *)&ADDPRIV_face->face_data;
     SET_TMAP(ADDPRIV_mat, ADDPRIV_vptr1data, ADDPRIV_vptr2data, ADDPRIV_vptr3data, ADDPRIV_vptr1, ADDPRIV_vptr2, ADDPRIV_vptr3, ADDPRIV_vptr1data,ADDPRIV_vptr2data,ADDPRIV_vptr3data,faceptr);
    }
}


void PR_LightFunc_FOG_LIGHTMAPA (void)
{
PR_FACE_DATA_REAL *faceptr;
PR_DWORD fogres;

  if (ADDPRIV_setup_data)
    {
     ADDPRIV_vptr1data->a = ADDPRIV_mat->a;
     ADDPRIV_vptr2data->a = ADDPRIV_mat->a;
     ADDPRIV_vptr3data->a = ADDPRIV_mat->a;

     fogres = AddAlphaFog1 ();
     if (fogres)
       ADDPRIV_vptr1data->dotproduct = -1;

     fogres = AddAlphaFog2 ();
     if (fogres)
       ADDPRIV_vptr2data->dotproduct = -1;

     fogres = AddAlphaFog3 ();
     if (fogres)
       ADDPRIV_vptr3data->dotproduct = -1;


     faceptr = (PR_FACE_DATA_REAL *)&ADDPRIV_face->face_data;
     SET_TMAP(ADDPRIV_mat, ADDPRIV_vptr1data, ADDPRIV_vptr2data, ADDPRIV_vptr3data, ADDPRIV_vptr1, ADDPRIV_vptr2, ADDPRIV_vptr3, ADDPRIV_vptr1data,ADDPRIV_vptr2data,ADDPRIV_vptr3data,faceptr);
    }


  if (ADDPRIV_setup_state)
    {
     if (last_material != ADDPRIV_mat)
       {
        PR_SelectTexture (ADDPRIV_mat->texture_number);
        last_material = ADDPRIV_mat;
       }

     if (last_method != 86)
       {
        guColorCombineFunction (GR_COLORCOMBINE_DECAL_TEXTURE);
        grTexCombineFunction (PR_CurrentTMU, GR_TEXTURECOMBINE_DECAL);
        grDepthBiasLevel (-1);

        guAlphaSource (GR_ALPHASOURCE_ITERATED_ALPHA);
        grAlphaBlendFunction (GR_BLEND_SRC_ALPHA,
                GR_BLEND_ONE, GR_BLEND_ONE, GR_BLEND_ZERO);
        grAlphaTestFunction(GR_CMP_ALWAYS);

        last_method = 86;
       }
    }
}



void PR_LightFunc_FOG_LIGHTMAPM (void)
{
PR_FACE_DATA_REAL *faceptr;
PR_DWORD fogres;

  if (ADDPRIV_setup_data)
    {
     ADDPRIV_vptr1data->a = ADDPRIV_mat->a;
     ADDPRIV_vptr2data->a = ADDPRIV_mat->a;
     ADDPRIV_vptr3data->a = ADDPRIV_mat->a;

     fogres = AddAlphaFog1 ();
     if (fogres)
       ADDPRIV_vptr1data->dotproduct = -1;

     fogres = AddAlphaFog2 ();
     if (fogres)
       ADDPRIV_vptr2data->dotproduct = -1;

     fogres = AddAlphaFog3 ();
     if (fogres)
       ADDPRIV_vptr3data->dotproduct = -1;


     faceptr = (PR_FACE_DATA_REAL *)&ADDPRIV_face->face_data;
     SET_TMAP(ADDPRIV_mat, ADDPRIV_vptr1data, ADDPRIV_vptr2data, ADDPRIV_vptr3data, ADDPRIV_vptr1, ADDPRIV_vptr2, ADDPRIV_vptr3, ADDPRIV_vptr1data,ADDPRIV_vptr2data,ADDPRIV_vptr3data,faceptr);
    }


  if (ADDPRIV_setup_state)
    {
     if (last_material != ADDPRIV_mat)
       {
        PR_SelectTexture (ADDPRIV_mat->texture_number);
        last_material = ADDPRIV_mat;
       }

     if (last_method != 95)
       {
        grDepthBiasLevel (-1);
	grColorCombine( GR_COMBINE_FUNCTION_SCALE_OTHER, GR_COMBINE_FACTOR_LOCAL,
                                        GR_COMBINE_LOCAL_ITERATED, GR_COMBINE_OTHER_TEXTURE, FXFALSE);
	grAlphaCombine( GR_COMBINE_FUNCTION_SCALE_OTHER, GR_COMBINE_FACTOR_LOCAL,
                                        GR_COMBINE_LOCAL_ITERATED, GR_COMBINE_OTHER_TEXTURE, FXFALSE);
	grTexCombine ( GR_TMU0, GR_COMBINE_FUNCTION_LOCAL, GR_COMBINE_FACTOR_NONE,
		GR_COMBINE_FUNCTION_LOCAL, GR_COMBINE_FACTOR_NONE, FXFALSE, FXFALSE);
        grAlphaBlendFunction (GR_BLEND_DST_COLOR, GR_BLEND_ZERO, 
                              GR_BLEND_ONE, GR_BLEND_ZERO);

        guColorCombineFunction (GR_COLORCOMBINE_DECAL_TEXTURE);
        grAlphaTestFunction(GR_CMP_ALWAYS);

        last_method = 95;
       }
    }
}



void PR_LightFunc_FOG_TRANSLUCENT_GOURAUD (void)
{
PR_REAL i;
PR_DWORD fogres;

  if (ADDPRIV_setup_data)
    {
     PR_WORKFACEFLAGS = !(ADDPRIV_face->flags & FFLAG_BACK_VISIBLE);

     if (ADDPRIV_vptr1data->dotproduct != ADDPRIV_material_num)
       {
        ADDPRIV_vptr1data->dotproduct = ADDPRIV_material_num;
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr1, ADDPRIV_vptr1data);

        i = ADDPRIV_facedatareal->col[0];          /* Add the precalced light */
        if (i > 0)
          {
           ADDPRIV_r += ADDPRIV_mat->r * i;
           ADDPRIV_g += ADDPRIV_mat->g * i;
           ADDPRIV_b += ADDPRIV_mat->b * i;
           CLAMP_RGB;
          }

        ADDPRIV_vptr1data->r = ADDPRIV_r;
        ADDPRIV_vptr1data->g = ADDPRIV_g;
        ADDPRIV_vptr1data->b = ADDPRIV_b;
        ADDPRIV_vptr1data->a = ADDPRIV_mat->a;
       }

     if (ADDPRIV_vptr2data->dotproduct != ADDPRIV_material_num)
       {
        ADDPRIV_vptr2data->dotproduct = ADDPRIV_material_num;
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr2, ADDPRIV_vptr2data);

        i = ADDPRIV_facedatareal->col[1];          /* Add the precalced light */
        if (i > 0)
          {
           ADDPRIV_r += ADDPRIV_mat->r * i;
           ADDPRIV_g += ADDPRIV_mat->g * i;
           ADDPRIV_b += ADDPRIV_mat->b * i;
           CLAMP_RGB;
          }

        ADDPRIV_vptr2data->r = ADDPRIV_r;
        ADDPRIV_vptr2data->g = ADDPRIV_g;
        ADDPRIV_vptr2data->b = ADDPRIV_b;
        ADDPRIV_vptr2data->a = ADDPRIV_mat->a;
       }

     if (ADDPRIV_vptr3data->dotproduct != ADDPRIV_material_num)
       {
        ADDPRIV_vptr3data->dotproduct = ADDPRIV_material_num;
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr3, ADDPRIV_vptr3data);

        i = ADDPRIV_facedatareal->col[2];          /* Add the precalced light */
        if (i > 0)
          {
           ADDPRIV_r += ADDPRIV_mat->r * i;
           ADDPRIV_g += ADDPRIV_mat->g * i;
           ADDPRIV_b += ADDPRIV_mat->b * i;
           CLAMP_RGB;
          }

        ADDPRIV_vptr3data->r = ADDPRIV_r;
        ADDPRIV_vptr3data->g = ADDPRIV_g;
        ADDPRIV_vptr3data->b = ADDPRIV_b;
        ADDPRIV_vptr3data->a = ADDPRIV_mat->a;
       }

     fogres = AddAlphaFog1 ();
     if (fogres)
       ADDPRIV_vptr1data->dotproduct = -1;

     fogres = AddAlphaFog2 ();
     if (fogres)
       ADDPRIV_vptr2data->dotproduct = -1;

     fogres = AddAlphaFog3 ();
     if (fogres)
       ADDPRIV_vptr3data->dotproduct = -1;
    }


  if (ADDPRIV_setup_state)
    {
     if (last_method != 87)
       {
        guColorCombineFunction (GR_COLORCOMBINE_ITRGB);

        grDepthBiasLevel (0);
        guAlphaSource( GR_ALPHASOURCE_ITERATED_ALPHA);
        grAlphaBlendFunction (GR_BLEND_SRC_ALPHA, GR_BLEND_ONE_MINUS_SRC_ALPHA, GR_BLEND_ONE, GR_BLEND_ZERO);
        grAlphaTestFunction(GR_CMP_ALWAYS);
        last_method = 87;
       }
    }
}


void PR_LightFunc_FOG_TRANSGOURTEX (void)
{
PR_REAL i;
PR_DWORD fogres;
PR_FACE_DATA_REAL *faceptr;

  if (ADDPRIV_setup_data)
    {
     PR_WORKFACEFLAGS = !(ADDPRIV_face->flags & FFLAG_BACK_VISIBLE);

     if (ADDPRIV_vptr1data->dotproduct != ADDPRIV_material_num)
       {
        ADDPRIV_vptr1data->dotproduct = ADDPRIV_material_num;
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr1, ADDPRIV_vptr1data);

        i = ADDPRIV_facedatareal->col[0];          /* Add the precalced light */
        if (i > 0)
          {
           ADDPRIV_r += ADDPRIV_mat->r * i;
           ADDPRIV_g += ADDPRIV_mat->g * i;
           ADDPRIV_b += ADDPRIV_mat->b * i;
           CLAMP_RGB;
          }

        ADDPRIV_vptr1data->r = ADDPRIV_r;
        ADDPRIV_vptr1data->g = ADDPRIV_g;
        ADDPRIV_vptr1data->b = ADDPRIV_b;
        ADDPRIV_vptr1data->a = ADDPRIV_mat->a;
       }

     if (ADDPRIV_vptr2data->dotproduct != ADDPRIV_material_num)
       {
        ADDPRIV_vptr2data->dotproduct = ADDPRIV_material_num;
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr2, ADDPRIV_vptr2data);

        i = ADDPRIV_facedatareal->col[1];          /* Add the precalced light */
        if (i > 0)
          {
           ADDPRIV_r += ADDPRIV_mat->r * i;
           ADDPRIV_g += ADDPRIV_mat->g * i;
           ADDPRIV_b += ADDPRIV_mat->b * i;
           CLAMP_RGB;
          }

        ADDPRIV_vptr2data->r = ADDPRIV_r;
        ADDPRIV_vptr2data->g = ADDPRIV_g;
        ADDPRIV_vptr2data->b = ADDPRIV_b;
        ADDPRIV_vptr2data->a = ADDPRIV_mat->a;
       }

     if (ADDPRIV_vptr3data->dotproduct != ADDPRIV_material_num)
       {
        ADDPRIV_vptr3data->dotproduct = ADDPRIV_material_num;
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr3, ADDPRIV_vptr3data);

        i = ADDPRIV_facedatareal->col[2];          /* Add the precalced light */
        if (i > 0)
          {
           ADDPRIV_r += ADDPRIV_mat->r * i;
           ADDPRIV_g += ADDPRIV_mat->g * i;
           ADDPRIV_b += ADDPRIV_mat->b * i;
           CLAMP_RGB;
          }

        ADDPRIV_vptr3data->r = ADDPRIV_r;
        ADDPRIV_vptr3data->g = ADDPRIV_g;
        ADDPRIV_vptr3data->b = ADDPRIV_b;
        ADDPRIV_vptr3data->a = ADDPRIV_mat->a;
       }

     fogres = AddAlphaFog1 ();
     if (fogres)
       ADDPRIV_vptr1data->dotproduct = -1;

     fogres = AddAlphaFog2 ();
     if (fogres)
       ADDPRIV_vptr2data->dotproduct = -1;

     fogres = AddAlphaFog3 ();
     if (fogres)
       ADDPRIV_vptr3data->dotproduct = -1;

     faceptr = (PR_FACE_DATA_REAL *)&ADDPRIV_face->face_data;
     SET_TMAP(ADDPRIV_mat, ADDPRIV_vptr1data, ADDPRIV_vptr2data, ADDPRIV_vptr3data, ADDPRIV_vptr1, ADDPRIV_vptr2, ADDPRIV_vptr3, ADDPRIV_vptr1data,ADDPRIV_vptr2data,ADDPRIV_vptr3data,faceptr);
    }


  if (ADDPRIV_setup_state)
    {
     if (last_material != ADDPRIV_mat)
       {
        PR_SelectTexture (ADDPRIV_mat->texture_number);
        last_material = ADDPRIV_mat;
       }

     if (last_method != 88)
       {
        guColorCombineFunction (GR_COLORCOMBINE_TEXTURE_TIMES_ITRGB);
        grTexCombineFunction (PR_CurrentTMU, GR_TEXTURECOMBINE_DECAL);

        grDepthBiasLevel (0);
        guAlphaSource( GR_ALPHASOURCE_ITERATED_ALPHA);
        grAlphaBlendFunction (GR_BLEND_SRC_ALPHA, GR_BLEND_ONE_MINUS_SRC_ALPHA, GR_BLEND_ONE, GR_BLEND_ZERO);
        grAlphaTestFunction(GR_CMP_ALWAYS);
        last_method = 88;
       }
    }
}



void PR_LightFunc_FOG_RGB_GOURAUD (void)
{
PR_DWORD packedrgb;
PR_DWORD fogres, fogged;

  if (ADDPRIV_setup_data)
    {
     packedrgb = *(PR_DWORD *)&ADDPRIV_facedatareal->col[0];
     ADDPRIV_vptr1data->r = (packedrgb)       & 0xFF;
     ADDPRIV_vptr1data->g = (packedrgb >> 8)  & 0xFF;
     ADDPRIV_vptr1data->b = (packedrgb >> 16) & 0xFF;
     ADDPRIV_vptr1data->a = 255.0f;

     packedrgb = *(PR_DWORD *)&ADDPRIV_facedatareal->col[1];
     ADDPRIV_vptr2data->r = (packedrgb)       & 0xFF;
     ADDPRIV_vptr2data->g = (packedrgb >> 8)  & 0xFF;
     ADDPRIV_vptr2data->b = (packedrgb >> 16) & 0xFF;
     ADDPRIV_vptr2data->a = 255.0f;

     packedrgb = *(PR_DWORD *)&ADDPRIV_facedatareal->col[2];
     ADDPRIV_vptr3data->r = (packedrgb)       & 0xFF;
     ADDPRIV_vptr3data->g = (packedrgb >> 8)  & 0xFF;
     ADDPRIV_vptr3data->b = (packedrgb >> 16) & 0xFF;
     ADDPRIV_vptr3data->a = 255.0f;

     fogres = AddAlphaFog1 ();
     if (fogres)
       ADDPRIV_vptr1data->dotproduct = -1;

     fogres = AddAlphaFog2 ();
     if (fogres)
       ADDPRIV_vptr2data->dotproduct = -1;

     fogres = AddAlphaFog3 ();
     if (fogres)
       ADDPRIV_vptr3data->dotproduct = -1;
    }


  if (ADDPRIV_setup_state)
    {
     if (last_method != 89)
       {
        guColorCombineFunction (GR_COLORCOMBINE_ITRGB);

        grDepthBiasLevel (0);
        grAlphaTestFunction (GR_CMP_ALWAYS);
        last_method = 89;
        last_fog = -1;
       }

     fogged = ((ADDPRIV_vptr1data->dotproduct == -1) ||
               (ADDPRIV_vptr2data->dotproduct == -1) ||
               (ADDPRIV_vptr3data->dotproduct == -1));
     if (fogged != last_fog)
       {
        if (fogged)
          {
           guAlphaSource(GR_ALPHASOURCE_ITERATED_ALPHA);
           grAlphaBlendFunction (GR_BLEND_SRC_ALPHA, GR_BLEND_ONE_MINUS_SRC_ALPHA, GR_BLEND_ONE, GR_BLEND_ZERO);
          }
        else
          grAlphaBlendFunction (GR_BLEND_ONE, GR_BLEND_ZERO,
                                GR_BLEND_ONE, GR_BLEND_ZERO);
        last_fog = fogged;
       }
    }
}



void PR_LightFunc_FOG_RGB_TEXTURED (void)
{
PR_FACE_DATA_REAL *faceptr;
PR_DWORD packedrgb;
PR_DWORD fogres, fogged;

  if (ADDPRIV_setup_data)
    {
     PR_WORKFACEFLAGS = !(ADDPRIV_face->flags & FFLAG_BACK_VISIBLE);

     packedrgb = *(PR_DWORD *)&ADDPRIV_facedatareal->col[0];
     ADDPRIV_vptr1data->r = (packedrgb)       & 0xFF;
     ADDPRIV_vptr1data->g = (packedrgb >> 8)  & 0xFF;
     ADDPRIV_vptr1data->b = (packedrgb >> 16) & 0xFF;
     ADDPRIV_vptr1data->a = 255.0f;

     packedrgb = *(PR_DWORD *)&ADDPRIV_facedatareal->col[1];
     ADDPRIV_vptr2data->r = (packedrgb)       & 0xFF;
     ADDPRIV_vptr2data->g = (packedrgb >> 8)  & 0xFF;
     ADDPRIV_vptr2data->b = (packedrgb >> 16) & 0xFF;
     ADDPRIV_vptr2data->a = 255.0f;

     packedrgb = *(PR_DWORD *)&ADDPRIV_facedatareal->col[2];
     ADDPRIV_vptr3data->r = (packedrgb)       & 0xFF;
     ADDPRIV_vptr3data->g = (packedrgb >> 8)  & 0xFF;
     ADDPRIV_vptr3data->b = (packedrgb >> 16) & 0xFF;
     ADDPRIV_vptr3data->a = 255.0f;

     fogres = AddAlphaFog1 ();
     if (fogres)
       ADDPRIV_vptr1data->dotproduct = -1;

     fogres = AddAlphaFog2 ();
     if (fogres)
       ADDPRIV_vptr2data->dotproduct = -1;

     fogres = AddAlphaFog3 ();
     if (fogres)
       ADDPRIV_vptr3data->dotproduct = -1;

     faceptr = (PR_FACE_DATA_REAL *)&ADDPRIV_face->face_data;
     SET_TMAP(ADDPRIV_mat, ADDPRIV_vptr1data, ADDPRIV_vptr2data, ADDPRIV_vptr3data, ADDPRIV_vptr1, ADDPRIV_vptr2, ADDPRIV_vptr3, ADDPRIV_vptr1data,ADDPRIV_vptr2data,ADDPRIV_vptr3data,faceptr);
    }


  if (ADDPRIV_setup_state)
    {
     if (last_material != ADDPRIV_mat)
       {
        PR_SelectTexture (ADDPRIV_mat->texture_number);
        last_material = ADDPRIV_mat;
       }

     if (last_method != 90)
       {
        guColorCombineFunction (GR_COLORCOMBINE_TEXTURE_TIMES_ITRGB);
        grTexCombineFunction (GR_TMU0, GR_TEXTURECOMBINE_DECAL);

        grDepthBiasLevel (0);
        grAlphaTestFunction (GR_CMP_ALWAYS);
        last_method = 90;
        last_fog = -1;
       }

     fogged = ((ADDPRIV_vptr1data->dotproduct == -1) ||
               (ADDPRIV_vptr2data->dotproduct == -1) ||
               (ADDPRIV_vptr3data->dotproduct == -1));
     if (fogged != last_fog)
       {
        if (fogged)
          {
           guAlphaSource(GR_ALPHASOURCE_ITERATED_ALPHA);
           grAlphaBlendFunction (GR_BLEND_SRC_ALPHA, GR_BLEND_ONE_MINUS_SRC_ALPHA, GR_BLEND_ONE, GR_BLEND_ZERO);
          }
        else
          grAlphaBlendFunction (GR_BLEND_ONE, GR_BLEND_ZERO,
                                GR_BLEND_ONE, GR_BLEND_ZERO);
        last_fog = fogged;
       }
    }
}



void PR_LightFunc_FOG_RGB_GOURAUD_DYN (void)
{
PR_DWORD packedrgb;
PR_DWORD fogres, fogged;

  if (ADDPRIV_setup_data)
    {
     packedrgb = *(PR_DWORD *)&ADDPRIV_facedatareal->col[0];
     ADDPRIV_vptr1data->r = (packedrgb)       & 0xFF;
     ADDPRIV_vptr1data->g = (packedrgb >> 8)  & 0xFF;
     ADDPRIV_vptr1data->b = (packedrgb >> 16) & 0xFF;
     ADDPRIV_vptr1data->a = 255.0f;

     packedrgb = *(PR_DWORD *)&ADDPRIV_facedatareal->col[1];
     ADDPRIV_vptr2data->r = (packedrgb)       & 0xFF;
     ADDPRIV_vptr2data->g = (packedrgb >> 8)  & 0xFF;
     ADDPRIV_vptr2data->b = (packedrgb >> 16) & 0xFF;
     ADDPRIV_vptr2data->a = 255.0f;

     packedrgb = *(PR_DWORD *)&ADDPRIV_facedatareal->col[2];
     ADDPRIV_vptr3data->r = (packedrgb)       & 0xFF;
     ADDPRIV_vptr3data->g = (packedrgb >> 8)  & 0xFF;
     ADDPRIV_vptr3data->b = (packedrgb >> 16) & 0xFF;
     ADDPRIV_vptr3data->a = 255.0f;

   
     {
        ADDPRIV_vptr1data->dotproduct = -1;
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr1, ADDPRIV_vptr1data);

        ADDPRIV_vptr1data->r += ADDPRIV_r;
        ADDPRIV_vptr1data->g += ADDPRIV_g;
        ADDPRIV_vptr1data->b += ADDPRIV_b;

        if (ADDPRIV_vptr1data->r > 255)
          ADDPRIV_vptr1data->r = 255;
        if (ADDPRIV_vptr1data->g > 255)
          ADDPRIV_vptr1data->g = 255;
        if (ADDPRIV_vptr1data->b > 255)
          ADDPRIV_vptr1data->b = 255;
       }


       {
        ADDPRIV_vptr2data->dotproduct = -1;
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr2, ADDPRIV_vptr2data);

        ADDPRIV_vptr2data->r += ADDPRIV_r;
        ADDPRIV_vptr2data->g += ADDPRIV_g;
        ADDPRIV_vptr2data->b += ADDPRIV_b;

        if (ADDPRIV_vptr2data->r > 255)
          ADDPRIV_vptr2data->r = 255;
        if (ADDPRIV_vptr2data->g > 255)
          ADDPRIV_vptr2data->g = 255;
        if (ADDPRIV_vptr2data->b > 255)
          ADDPRIV_vptr2data->b = 255;
       }

       {
        ADDPRIV_vptr3data->dotproduct = -1;
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr3, ADDPRIV_vptr3data);

        ADDPRIV_vptr3data->r += ADDPRIV_r;
        ADDPRIV_vptr3data->g += ADDPRIV_g;
        ADDPRIV_vptr3data->b += ADDPRIV_b;

        if (ADDPRIV_vptr3data->r > 255)
          ADDPRIV_vptr3data->r = 255;
        if (ADDPRIV_vptr3data->g > 255)
          ADDPRIV_vptr3data->g = 255;
        if (ADDPRIV_vptr3data->b > 255)
          ADDPRIV_vptr3data->b = 255;
       }

     fogres = AddAlphaFog1 ();
     if (fogres)
       ADDPRIV_vptr1data->dotproduct = -1;

     fogres = AddAlphaFog2 ();
     if (fogres)
       ADDPRIV_vptr2data->dotproduct = -1;

     fogres = AddAlphaFog3 ();
     if (fogres)
       ADDPRIV_vptr3data->dotproduct = -1;
    }


  if (ADDPRIV_setup_state)
    {
     if (last_method != 91)
       {
        guColorCombineFunction (GR_COLORCOMBINE_ITRGB);

        grDepthBiasLevel (0);
        grAlphaTestFunction (GR_CMP_ALWAYS);
        last_method = 92;
        last_fog = -1;
       }

     fogged = ((ADDPRIV_vptr1data->dotproduct == -1) ||
               (ADDPRIV_vptr2data->dotproduct == -1) ||
               (ADDPRIV_vptr3data->dotproduct == -1));
     if (fogged != last_fog)
       {
        if (fogged)
          {
           guAlphaSource(GR_ALPHASOURCE_ITERATED_ALPHA);
           grAlphaBlendFunction (GR_BLEND_SRC_ALPHA, GR_BLEND_ONE_MINUS_SRC_ALPHA, GR_BLEND_ONE, GR_BLEND_ZERO);
          }
        else
          grAlphaBlendFunction (GR_BLEND_ONE, GR_BLEND_ZERO,
                                GR_BLEND_ONE, GR_BLEND_ZERO);
        last_fog = fogged;
       }
    }
}



void PR_LightFunc_FOG_RGB_TEXTURED_DYN (void)
{
PR_FACE_DATA_REAL *faceptr;
PR_DWORD packedrgb;
PR_DWORD fogres, fogged;

  if (ADDPRIV_setup_data)
    {
     PR_WORKFACEFLAGS = !(ADDPRIV_face->flags & FFLAG_BACK_VISIBLE);

     packedrgb = *(PR_DWORD *)&ADDPRIV_facedatareal->col[0];
     ADDPRIV_vptr1data->r = (packedrgb)       & 0xFF;
     ADDPRIV_vptr1data->g = (packedrgb >> 8)  & 0xFF;
     ADDPRIV_vptr1data->b = (packedrgb >> 16) & 0xFF;
     ADDPRIV_vptr1data->a = 255.0f;

     packedrgb = *(PR_DWORD *)&ADDPRIV_facedatareal->col[1];
     ADDPRIV_vptr2data->r = (packedrgb)       & 0xFF;
     ADDPRIV_vptr2data->g = (packedrgb >> 8)  & 0xFF;
     ADDPRIV_vptr2data->b = (packedrgb >> 16) & 0xFF;
     ADDPRIV_vptr2data->a = 255.0f;

     packedrgb = *(PR_DWORD *)&ADDPRIV_facedatareal->col[2];
     ADDPRIV_vptr3data->r = (packedrgb)       & 0xFF;
     ADDPRIV_vptr3data->g = (packedrgb >> 8)  & 0xFF;
     ADDPRIV_vptr3data->b = (packedrgb >> 16) & 0xFF;
     ADDPRIV_vptr3data->a = 255.0f;

       {
        ADDPRIV_vptr1data->dotproduct = -1;
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr1, ADDPRIV_vptr1data);

        ADDPRIV_vptr1data->r += ADDPRIV_r;
        ADDPRIV_vptr1data->g += ADDPRIV_g;
        ADDPRIV_vptr1data->b += ADDPRIV_b;

        if (ADDPRIV_vptr1data->r > 255)
          ADDPRIV_vptr1data->r = 255;
        if (ADDPRIV_vptr1data->g > 255)
          ADDPRIV_vptr1data->g = 255;
        if (ADDPRIV_vptr1data->b > 255)
          ADDPRIV_vptr1data->b = 255;
       }

       {
        ADDPRIV_vptr2data->dotproduct = -1;
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr2, ADDPRIV_vptr2data);

        ADDPRIV_vptr2data->r += ADDPRIV_r;
        ADDPRIV_vptr2data->g += ADDPRIV_g;
        ADDPRIV_vptr2data->b += ADDPRIV_b;

        if (ADDPRIV_vptr2data->r > 255)
          ADDPRIV_vptr2data->r = 255;
        if (ADDPRIV_vptr2data->g > 255)
          ADDPRIV_vptr2data->g = 255;
        if (ADDPRIV_vptr2data->b > 255)
          ADDPRIV_vptr2data->b = 255;
       }

       {
        ADDPRIV_vptr3data->dotproduct = -1;
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr3, ADDPRIV_vptr3data);

        ADDPRIV_vptr3data->r += ADDPRIV_r;
        ADDPRIV_vptr3data->g += ADDPRIV_g;
        ADDPRIV_vptr3data->b += ADDPRIV_b;

        if (ADDPRIV_vptr3data->r > 255)
          ADDPRIV_vptr3data->r = 255;
        if (ADDPRIV_vptr3data->g > 255)
          ADDPRIV_vptr3data->g = 255;
        if (ADDPRIV_vptr3data->b > 255)
          ADDPRIV_vptr3data->b = 255;
       }

     fogres = AddAlphaFog1 ();
     if (fogres)
       ADDPRIV_vptr1data->dotproduct = -1;

     fogres = AddAlphaFog2 ();
     if (fogres)
       ADDPRIV_vptr2data->dotproduct = -1;

     fogres = AddAlphaFog3 ();
     if (fogres)
       ADDPRIV_vptr3data->dotproduct = -1;

     faceptr = (PR_FACE_DATA_REAL *)&ADDPRIV_face->face_data;
     SET_TMAP(ADDPRIV_mat, ADDPRIV_vptr1data, ADDPRIV_vptr2data, ADDPRIV_vptr3data, ADDPRIV_vptr1, ADDPRIV_vptr2, ADDPRIV_vptr3, ADDPRIV_vptr1data,ADDPRIV_vptr2data,ADDPRIV_vptr3data,faceptr);
    }


  if (ADDPRIV_setup_state)
    {
     if (last_material != ADDPRIV_mat)
       {
        PR_SelectTexture (ADDPRIV_mat->texture_number);
        last_material = ADDPRIV_mat;
       }

     if (last_method != 92)
       {
        guColorCombineFunction (GR_COLORCOMBINE_TEXTURE_TIMES_ITRGB);
        grTexCombineFunction (GR_TMU0, GR_TEXTURECOMBINE_DECAL);

        grDepthBiasLevel (0);
        grAlphaTestFunction (GR_CMP_ALWAYS);
        last_method = 92;
        last_fog = -1;
       }

     fogged = ((ADDPRIV_vptr1data->dotproduct == -1) ||
               (ADDPRIV_vptr2data->dotproduct == -1) ||
               (ADDPRIV_vptr3data->dotproduct == -1));
     if (fogged != last_fog)
       {
        if (fogged)
          {
           guAlphaSource(GR_ALPHASOURCE_ITERATED_ALPHA);
           grAlphaBlendFunction (GR_BLEND_SRC_ALPHA, GR_BLEND_ONE_MINUS_SRC_ALPHA, GR_BLEND_ONE, GR_BLEND_ZERO);
          }
        else
          grAlphaBlendFunction (GR_BLEND_ONE, GR_BLEND_ZERO,
                                GR_BLEND_ONE, GR_BLEND_ZERO);
        last_fog = fogged;
       }
    }
}


void PR_LightFunc_WATER_SPECIAL (void) 
{
PR_FACE_DATA_REAL *faceptr;
PR_REAL i;


  if (ADDPRIV_setup_state)
    {
     if (last_material != ADDPRIV_mat)
       {
        PR_SelectTexture (ADDPRIV_mat->texture_number);
        last_material = ADDPRIV_mat;
       }

     if (last_method != 93)
       {
        guColorCombineFunction (GR_COLORCOMBINE_TEXTURE_TIMES_ITRGB);
        grTexCombineFunction (GR_TMU0, GR_TEXTURECOMBINE_DECAL);

        grDepthBiasLevel (0);
        guAlphaSource( GR_ALPHASOURCE_ITERATED_ALPHA);
        grAlphaBlendFunction (GR_BLEND_SRC_ALPHA, GR_BLEND_ONE_MINUS_SRC_ALPHA, GR_BLEND_ONE, GR_BLEND_ZERO);
        grAlphaTestFunction (GR_CMP_ALWAYS);

        last_method = 93;
       }
  }

  if (ADDPRIV_setup_data)
    {
     PR_WORKFACEFLAGS = !(ADDPRIV_face->flags & FFLAG_BACK_VISIBLE);
     if (ADDPRIV_vptr1data->dotproduct != ADDPRIV_material_num)
       {
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr1, ADDPRIV_vptr1data);

        i = ADDPRIV_facedatareal->col[0];          /* Add the precalced light */
        if (i > 0)
          {
           ADDPRIV_r += ADDPRIV_mat->r * i;
           ADDPRIV_g += ADDPRIV_mat->g * i;
           ADDPRIV_b += ADDPRIV_mat->b * i;
           CLAMP_RGB;
           ADDPRIV_vptr1data->dotproduct = -1;
          }
        else
          ADDPRIV_vptr1data->dotproduct = ADDPRIV_material_num;
   
        ADDPRIV_vptr1data->r = ADDPRIV_r;
        ADDPRIV_vptr1data->g = ADDPRIV_g;
        ADDPRIV_vptr1data->b = ADDPRIV_b;

        if (ADDPRIV_face->backmaterial & 1)
          ADDPRIV_vptr1data->a = 255.0f;
        else
          ADDPRIV_vptr1data->a = ADDPRIV_mat->a;
       }

     if (ADDPRIV_vptr2data->dotproduct != ADDPRIV_material_num)
       {
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr2, ADDPRIV_vptr2data);
   
        i = ADDPRIV_facedatareal->col[1];          /* Add the precalced light */
        if (i > 0)
          {
           ADDPRIV_r += ADDPRIV_mat->r * i;
           ADDPRIV_g += ADDPRIV_mat->g * i;
           ADDPRIV_b += ADDPRIV_mat->b * i;
           CLAMP_RGB;
           ADDPRIV_vptr2data->dotproduct = -1;
          }
        else
          ADDPRIV_vptr2data->dotproduct = ADDPRIV_material_num;
   
        ADDPRIV_vptr2data->r = ADDPRIV_r;
        ADDPRIV_vptr2data->g = ADDPRIV_g;
        ADDPRIV_vptr2data->b = ADDPRIV_b;

        if (ADDPRIV_face->backmaterial & 4)
          ADDPRIV_vptr2data->a = 255.0f;
        else
          ADDPRIV_vptr2data->a = ADDPRIV_mat->a;
       }

     if (ADDPRIV_vptr3data->dotproduct != ADDPRIV_material_num)
       {
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr3, ADDPRIV_vptr3data);
   
        i = ADDPRIV_facedatareal->col[2];          /* Add the precalced light */
        if (i > 0)
          {
           ADDPRIV_r += ADDPRIV_mat->r * i;
           ADDPRIV_g += ADDPRIV_mat->g * i;
           ADDPRIV_b += ADDPRIV_mat->b * i;
           CLAMP_RGB;
           ADDPRIV_vptr3data->dotproduct = -1;
          }
        else
          ADDPRIV_vptr3data->dotproduct = ADDPRIV_material_num;
   
        ADDPRIV_vptr3data->r = ADDPRIV_r;
        ADDPRIV_vptr3data->g = ADDPRIV_g;
        ADDPRIV_vptr3data->b = ADDPRIV_b;

        if (ADDPRIV_face->backmaterial & 2)
          ADDPRIV_vptr3data->a = 255.0f;
        else
          ADDPRIV_vptr3data->a = ADDPRIV_mat->a;
       }

     faceptr = (PR_FACE_DATA_REAL *)&ADDPRIV_face->face_data;
     SET_TMAP(ADDPRIV_mat, ADDPRIV_vptr1data, ADDPRIV_vptr2data, ADDPRIV_vptr3data, ADDPRIV_vptr1, ADDPRIV_vptr2, ADDPRIV_vptr3, ADDPRIV_vptr1data,ADDPRIV_vptr2data,ADDPRIV_vptr3data,faceptr);
    }
}


void PR_LightFunc_FOG_WATER_SPECIAL (void) 
{
PR_FACE_DATA_REAL *faceptr;
PR_REAL i;


  if (ADDPRIV_setup_state)
    {
     if (last_material != ADDPRIV_mat)
       {
        PR_SelectTexture (ADDPRIV_mat->texture_number);
        last_material = ADDPRIV_mat;
       }

     if (last_method != 94)
       {
        guColorCombineFunction (GR_COLORCOMBINE_TEXTURE_TIMES_ITRGB);
        grTexCombineFunction (GR_TMU0, GR_TEXTURECOMBINE_DECAL);

        grDepthBiasLevel (0);
        guAlphaSource( GR_ALPHASOURCE_ITERATED_ALPHA);
        grAlphaBlendFunction (GR_BLEND_SRC_ALPHA, GR_BLEND_ONE_MINUS_SRC_ALPHA, GR_BLEND_ONE, GR_BLEND_ZERO);
        grAlphaTestFunction (GR_CMP_ALWAYS);

        last_method = 94;
       }
    }
   

  if (ADDPRIV_setup_data)
    {
     PR_WORKFACEFLAGS = !(ADDPRIV_face->flags & FFLAG_BACK_VISIBLE);
     if (ADDPRIV_vptr1data->dotproduct != ADDPRIV_material_num)
       {
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr1, ADDPRIV_vptr1data);

        i = ADDPRIV_facedatareal->col[0];          /* Add the precalced light */
        if (i > 0)
          {
           ADDPRIV_r += ADDPRIV_mat->r * i;
           ADDPRIV_g += ADDPRIV_mat->g * i;
           ADDPRIV_b += ADDPRIV_mat->b * i;
           CLAMP_RGB;
           ADDPRIV_vptr1data->dotproduct = -1;
          }
        else
          ADDPRIV_vptr1data->dotproduct = ADDPRIV_material_num;
   
        ADDPRIV_vptr1data->r = ADDPRIV_r;
        ADDPRIV_vptr1data->g = ADDPRIV_g;
        ADDPRIV_vptr1data->b = ADDPRIV_b;

        if (ADDPRIV_face->backmaterial & 1)
          ADDPRIV_vptr1data->a = 255.0f;
        else
          ADDPRIV_vptr1data->a = ADDPRIV_mat->a;

        if (AddAlphaFog1 ())
          ADDPRIV_vptr1data->dotproduct = -1;
       }

     if (ADDPRIV_vptr2data->dotproduct != ADDPRIV_material_num)
       {
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr2, ADDPRIV_vptr2data);
   
        i = ADDPRIV_facedatareal->col[1];          /* Add the precalced light */
        if (i > 0)
          {
           ADDPRIV_r += ADDPRIV_mat->r * i;
           ADDPRIV_g += ADDPRIV_mat->g * i;
           ADDPRIV_b += ADDPRIV_mat->b * i;
           CLAMP_RGB;
           ADDPRIV_vptr2data->dotproduct = -1;
          }
        else
          ADDPRIV_vptr2data->dotproduct = ADDPRIV_material_num;
   
        ADDPRIV_vptr2data->r = ADDPRIV_r;
        ADDPRIV_vptr2data->g = ADDPRIV_g;
        ADDPRIV_vptr2data->b = ADDPRIV_b;

        if (ADDPRIV_face->backmaterial & 4)
          ADDPRIV_vptr2data->a = 255.0f;
        else
          ADDPRIV_vptr2data->a = ADDPRIV_mat->a;

        if (AddAlphaFog2 ())
          ADDPRIV_vptr2data->dotproduct = -1;
       }

     if (ADDPRIV_vptr3data->dotproduct != ADDPRIV_material_num)
       {
        PR_CalcVertexLighting3Dfx (ADDPRIV_vptr3, ADDPRIV_vptr3data);
   
        i = ADDPRIV_facedatareal->col[2];          /* Add the precalced light */
        if (i > 0)
          {
           ADDPRIV_r += ADDPRIV_mat->r * i;
           ADDPRIV_g += ADDPRIV_mat->g * i;
           ADDPRIV_b += ADDPRIV_mat->b * i;
           CLAMP_RGB;
           ADDPRIV_vptr3data->dotproduct = -1;
          }
        else
          ADDPRIV_vptr3data->dotproduct = ADDPRIV_material_num;
   
        ADDPRIV_vptr3data->r = ADDPRIV_r;
        ADDPRIV_vptr3data->g = ADDPRIV_g;
        ADDPRIV_vptr3data->b = ADDPRIV_b;

        if (ADDPRIV_face->backmaterial & 2)
          ADDPRIV_vptr3data->a = 255.0f;
        else
          ADDPRIV_vptr3data->a = ADDPRIV_mat->a;

        if (AddAlphaFog3 ())
          ADDPRIV_vptr3data->dotproduct = -1;
       }

     faceptr = (PR_FACE_DATA_REAL *)&ADDPRIV_face->face_data;
     SET_TMAP(ADDPRIV_mat, ADDPRIV_vptr1data, ADDPRIV_vptr2data, ADDPRIV_vptr3data, ADDPRIV_vptr1, ADDPRIV_vptr2, ADDPRIV_vptr3, ADDPRIV_vptr1data,ADDPRIV_vptr2data,ADDPRIV_vptr3data,faceptr);
    }
}


/* Rendering methods */

#define L_WIRE            PR_LightFunc_WIREFRAME
#define L_CFLAT           PR_LightFunc_CFLAT
#define L_FLAT            PR_LightFunc_FLAT
#define L_CGOURAUD        PR_LightFunc_CGOURAUD
#define L_GOURAUD         PR_LightFunc_GOURAUD
#define L_DECAL           PR_LightFunc_DECAL
#define L_XDECAL          PR_LightFunc_XDECAL
#define L_CFLATTEXT       PR_LightFunc_CFLATTEXT
#define L_XCFLATTEXT      PR_LightFunc_XCFLATTEXT
#define L_FLATTEXT        PR_LightFunc_FLATTEXT
#define L_XFLATTEXT       PR_LightFunc_XFLATTEXT
#define L_CGOURAUDTEXT    PR_LightFunc_CGOURAUDTEXT
#define L_XGOURAUDTEXT    PR_LightFunc_XGOURAUDTEXT
#define L_XCGOURAUDTEXT   PR_LightFunc_XCGOURAUDTEXT
#define L_GOURAUDTEXT     PR_LightFunc_GOURAUDTEXT
#define L_TRANSTEXT       PR_LightFunc_TRANSTEXT
#define L_XTRANSTEXT      PR_LightFunc_XTRANSTEXT
#define L_LIGHTMAPA       PR_LightFunc_LIGHTMAPA
#define L_LIGHTMAPM       PR_LightFunc_LIGHTMAPM

#define L_TRANSGOUR       PR_LightFunc_TRANSGOUR
#define L_TRANSGOURTEX    PR_LightFunc_TRANSGOURTEX
#define L_RGBGOUR         PR_LightFunc_RGBGOUR
#define L_RGBTEX          PR_LightFunc_RGBTEX
#define L_RGBGOUR_DYN     PR_LightFunc_RGBGOUR_DYN
#define L_RGBTEX_DYN      PR_LightFunc_RGBTEX_DYN
#define L_FOG_GOUR        PR_LightFunc_FOG_GOURAUD
#define L_FOG_TEX         PR_LightFunc_FOG_DECAL
#define L_FOG_XTEX        PR_LightFunc_FOG_XDECAL
#define L_FOG_FLATTEX     PR_LightFunc_FOG_FLAT_TEXTURED
#define L_FOG_XFLATTEX    PR_LightFunc_FOG_XFLAT_TEXTURED
#define L_FOG_GOURTEX     PR_LightFunc_FOG_GOURAUD_TEXTURED
#define L_FOG_XGOURTEX    PR_LightFunc_FOG_XGOURAUD_TEXTURED
#define L_FOG_TRANSTEX    PR_LightFunc_FOG_TRANSLUCENT_TEXTURED
#define L_FOG_XTRANSTEX   PR_LightFunc_FOG_XTRANSLUCENT_TEXTURED
#define L_FOG_LIGHTMAPA   PR_LightFunc_FOG_LIGHTMAPA
#define L_FOG_LIGHTMAPM   PR_LightFunc_FOG_LIGHTMAPM
#define L_FOG_TRANSGOUR   PR_LightFunc_FOG_TRANSLUCENT_GOURAUD
#define L_FOG_TRANSGOURTEX PR_LightFunc_FOG_TRANSGOURTEX
#define L_FOG_RGBGOUR     PR_LightFunc_FOG_RGB_GOURAUD
#define L_FOG_RGBTEX      PR_LightFunc_FOG_RGB_TEXTURED
#define L_FOG_RGBGOUR_DYN PR_LightFunc_FOG_RGB_GOURAUD_DYN
#define L_FOG_RGBTEX_DYN  PR_LightFunc_FOG_RGB_TEXTURED_DYN
#define L_WATER_SPEC      PR_LightFunc_WATER_SPECIAL
#define L_FOG_WATER_SPEC  PR_LightFunc_FOG_WATER_SPECIAL



void (*PR_LightMethod[256])();
PR_DWORD PR_MethodFlags[256];

void PR_RegisterRenderMethod (int method_number, void (*MethodProc)(),
                PR_DWORD flags)
/* Sets the routine used for initializing the state data for a particular
   rendering method. This is used to add new rendering methods to the
   ones existing in Power Render.  */
{
  if (method_number < 256)
    {
     PR_LightMethod[method_number] = MethodProc;
     PR_MethodFlags[method_number] = flags;
    }
}


void PR_InitializeRenderMethods (void)
/* Sets the initial rendering methods array */
{
PR_DWORD i;

  for (i = 0; i < 256; i++)
    PR_RegisterRenderMethod (i, NULL, 0);

  PR_RegisterRenderMethod(1,  L_WIRE, 0);         // Wireframe
  PR_RegisterRenderMethod(2,  L_CFLAT, 0);        // T_CFLAT
  PR_RegisterRenderMethod(3,  L_FLAT, 0);         // T_FLAT
  PR_RegisterRenderMethod(4,  L_CGOURAUD, 0);     // T_CGOURAUD
  PR_RegisterRenderMethod(5,  L_GOURAUD, 0);      // T_GOURAUD
  PR_RegisterRenderMethod(6,  L_DECAL, 0);        // T_LTEXTURED
  PR_RegisterRenderMethod(7,  L_XDECAL, 1);       // T_LXTEXTURED
  PR_RegisterRenderMethod(8,  L_DECAL, 0);        // T_TEXTURED
  PR_RegisterRenderMethod(9,  L_DECAL, 0);        // T_WTEXTURED
  PR_RegisterRenderMethod(10, L_XDECAL, 1);       // T_XTEXTURED
  PR_RegisterRenderMethod(11, L_XDECAL, 1);       // T_XWTEXTURED
  PR_RegisterRenderMethod(12, L_CFLATTEXT, 0);    // T_LCFLAT_TEXTURED
  PR_RegisterRenderMethod(13, L_XCFLATTEXT, 1);   // T_LXCFLAT_TEXTURED
  PR_RegisterRenderMethod(14, L_CFLATTEXT, 0);    // T_CFLAT_TEXTURED
  PR_RegisterRenderMethod(15, L_CFLATTEXT, 0);    // T_WCFLAT_TEXTURED
  PR_RegisterRenderMethod(16, L_XCFLATTEXT, 1);   // T_XCFLAT_TEXTURED
  PR_RegisterRenderMethod(17, L_XCFLATTEXT, 1);   // T_XWCFLAT_TEXTURED
  PR_RegisterRenderMethod(18, L_FLATTEXT, 0);     // T_FLAT_TEXTURED
  PR_RegisterRenderMethod(19, L_FLATTEXT, 0);     // T_WFLAT_TEXTURED
  PR_RegisterRenderMethod(20, L_XFLATTEXT, 1);    // T_XFLAT_TEXTURED
  PR_RegisterRenderMethod(21, L_XFLATTEXT, 1);    // T_XWFLAT_TEXTURED
  PR_RegisterRenderMethod(22, L_FLATTEXT, 0);     // T_LFLAT_TEXTURED
  PR_RegisterRenderMethod(23, L_XFLATTEXT, 1);    // T_LXFLAT_TEXTURED
  PR_RegisterRenderMethod(24, L_CGOURAUDTEXT, 0); // T_LCGOURAUD_TEXTURED
  PR_RegisterRenderMethod(25, L_XCGOURAUDTEXT, 1);// T_LXCGOURAUD_TEXTURED
  PR_RegisterRenderMethod(26, L_CGOURAUDTEXT, 0); // T_CGOURAUD_TEXTURED8
  PR_RegisterRenderMethod(27, L_CGOURAUDTEXT, 0); // T_WCGOURAUD_TEXTURED8
  PR_RegisterRenderMethod(28, L_XCGOURAUDTEXT, 1);// T_XCGOURAUD_TEXTURED8
  PR_RegisterRenderMethod(29, L_XCGOURAUDTEXT, 1);// T_XWCGOURAUD_TEXTURED8
  PR_RegisterRenderMethod(30, L_GOURAUDTEXT, 0);  // T_LGOURAUD_TEXTURED
  PR_RegisterRenderMethod(31, L_XGOURAUDTEXT, 1); // T_LXGOURAUD_TEXTURED
  PR_RegisterRenderMethod(32, L_GOURAUDTEXT, 0);  // T_GOURAUD_TEXTURED8
  PR_RegisterRenderMethod(33, L_GOURAUDTEXT, 0);  // T_WGOURAUD_TEXTURED8
  PR_RegisterRenderMethod(34, L_XGOURAUDTEXT, 1); // T_XGOURAUD_TEXTURED8
  PR_RegisterRenderMethod(35, L_XGOURAUDTEXT, 1); // T_XWGOURAUD_TEXTURED8
  PR_RegisterRenderMethod(36, L_TRANSTEXT, 1);    // T_LTRANSLUCENT_TEXTURED    
  PR_RegisterRenderMethod(37, L_XTRANSTEXT, 1);   // T_LXTRANSLUCENT_TEXTURED    
  PR_RegisterRenderMethod(38, L_TRANSTEXT, 1);    // T_TRANSLUCENT_TEXTURED    
  PR_RegisterRenderMethod(39, L_TRANSTEXT, 1);    // T_WTRANSLUCENT_TEXTURED    
  PR_RegisterRenderMethod(40, L_XTRANSTEXT, 1);   // T_XTRANSLUCENT_TEXTURED    
  PR_RegisterRenderMethod(41, L_XTRANSTEXT, 1);   // T_XWTRANSLUCENT_TEXTURED    
  PR_RegisterRenderMethod(42, L_CGOURAUD, 0);     // T_PCGOURAUD
  PR_RegisterRenderMethod(43, L_GOURAUD, 0);      // T_PGOURAUD
  PR_RegisterRenderMethod(44, L_DECAL, 0);        // T_PTEXTURED
  PR_RegisterRenderMethod(45, L_DECAL, 0);        // T_PWTEXTURED
  PR_RegisterRenderMethod(46, L_XDECAL, 1);       // T_PXTEXTURED
  PR_RegisterRenderMethod(47, L_XDECAL, 1);       // T_PXWTEXTURED
  PR_RegisterRenderMethod(48, L_CFLATTEXT, 0);    // T_PCFLAT_TEXTURED
  PR_RegisterRenderMethod(49, L_CFLATTEXT, 0);    // T_PWCFLAT_TEXTURED
  PR_RegisterRenderMethod(50, L_XCFLATTEXT, 1);   // T_PXCFLAT_TEXTURED
  PR_RegisterRenderMethod(51, L_XCFLATTEXT, 1);   // T_PXWCFLAT_TEXTURED
  PR_RegisterRenderMethod(52, L_FLATTEXT, 0);     // T_PFLAT_TEXTURED
  PR_RegisterRenderMethod(53, L_FLATTEXT, 0);     // T_PWFLAT_TEXTURED
  PR_RegisterRenderMethod(54, L_XFLATTEXT, 1);    // T_PXFLAT_TEXTURED
  PR_RegisterRenderMethod(55, L_XFLATTEXT, 1);    // T_PXWFLAT_TEXTURED
  PR_RegisterRenderMethod(56, L_CGOURAUDTEXT, 0); // T_PCGOURAUD_TEXTURED
  PR_RegisterRenderMethod(57, L_CGOURAUDTEXT, 0); // T_PWCGOURAUD_TEXTURED
  PR_RegisterRenderMethod(58, L_XCGOURAUDTEXT, 1);// T_PXCGOURAUD_TEXTURED
  PR_RegisterRenderMethod(59, L_XCGOURAUDTEXT, 1);// T_PXWCGOURAUD_TEXTURED
  PR_RegisterRenderMethod(60, L_GOURAUDTEXT, 0);  // T_PGOURAUD_TEXTURED
  PR_RegisterRenderMethod(61, L_GOURAUDTEXT, 0);  // T_PWGOURAUD_TEXTURED
  PR_RegisterRenderMethod(62, L_XGOURAUDTEXT, 1); // T_PXGOURAUD_TEXTURED
  PR_RegisterRenderMethod(63, L_XGOURAUDTEXT, 1); // T_PXWGOURAUD_TEXTURED
  PR_RegisterRenderMethod(64, L_TRANSTEXT, 1);    // T_PTRANSLUCENT_TEXTURED    
  PR_RegisterRenderMethod(65, L_TRANSTEXT, 1);    // T_PWTRANSLUCENT_TEXTURED    
  PR_RegisterRenderMethod(66, L_XTRANSTEXT, 1);   // T_PXTRANSLUCENT_TEXTURED    
  PR_RegisterRenderMethod(67, L_XTRANSTEXT, 1);   // T_PXWTRANSLUCENT_TEXTURED    
  PR_RegisterRenderMethod(68, L_FLAT, 1);         // T_SOURCETRANSFORM
  PR_RegisterRenderMethod(69, L_LIGHTMAPA, 1);    // T_LIGHTMAP_ADD
  PR_RegisterRenderMethod(70, L_LIGHTMAPM, 1);    // T_LIGHTMAP_TIMES

  /* New for PR 2.6 */
  PR_RegisterRenderMethod(71, L_TRANSGOUR, 1);    // T_TRANSLUCENT_GOURAUD
  PR_RegisterRenderMethod(72, L_TRANSGOURTEX, 1); // T_TRANSLUCENT_GOURAUD_TEXTURED
  PR_RegisterRenderMethod(73, L_RGBGOUR, 0);      // T_RGB_GOURAUD
  PR_RegisterRenderMethod(74, L_RGBTEX, 0);       // T_RGB_TEXTURED
  PR_RegisterRenderMethod(75, L_RGBGOUR_DYN, 0);  // T_RGB_GOURAUD_DYN
  PR_RegisterRenderMethod(76, L_RGBTEX_DYN, 0);   // T_RGB_TEXTURED_DYN
  PR_RegisterRenderMethod(77, L_FOG_GOUR, 0);     // T_FOG_GOURAUD
  PR_RegisterRenderMethod(78, L_FOG_TEX, 0);      // T_FOG_TEXTURED
  PR_RegisterRenderMethod(79, L_FOG_XTEX, 1);     // T_FOG_XTEXTURED
  PR_RegisterRenderMethod(80, L_FOG_FLATTEX, 0);  // T_FOG_FLAT_TEXTURED
  PR_RegisterRenderMethod(81, L_FOG_XFLATTEX, 1); // T_FOG_XFLAT_TEXTURED
  PR_RegisterRenderMethod(82, L_FOG_GOURTEX, 0);  // T_FOG_GOURAUD_TEXTURED
  PR_RegisterRenderMethod(83, L_FOG_XGOURTEX, 1); // T_FOG_XGOURAUD_TEXTURED
  PR_RegisterRenderMethod(84, L_FOG_TRANSTEX, 0); // T_FOG_TRANSLUCENT_TEXTURED
  PR_RegisterRenderMethod(85, L_FOG_XTRANSTEX, 1);// T_FOG_XTRANSLUCENT_TEXTURED
  PR_RegisterRenderMethod(86, L_FOG_LIGHTMAPA, 1);// T_FOG_LIGHTMAPA
  PR_RegisterRenderMethod(87, L_FOG_TRANSGOUR, 1);// T_FOG_TRANSLUCENT_GOURAUD
  PR_RegisterRenderMethod(88, L_FOG_TRANSGOURTEX, 1);// T_FOG_TRANSLUCENT_GOURAUD_TEXTURED
  PR_RegisterRenderMethod(89, L_FOG_RGBGOUR, 0);     // T_FOG_RGB_GOURAUD
  PR_RegisterRenderMethod(90, L_FOG_RGBTEX, 0);      // T_FOG_RGB_TEXTURED
  PR_RegisterRenderMethod(91, L_FOG_RGBGOUR_DYN, 0); // T_FOG_RGB_GOURAUD_DYN
  PR_RegisterRenderMethod(92, L_FOG_RGBTEX_DYN, 0);  // T_FOG_RGB_TEXTURED_DYN
  PR_RegisterRenderMethod(93, L_WATER_SPEC, 1);      // T_WATER_SPECIAL
  PR_RegisterRenderMethod(94, L_FOG_WATER_SPEC, 1);  // T_FOG_WATER_SPECIAL
  PR_RegisterRenderMethod(95, L_FOG_LIGHTMAPM, 1);   // T_FOG_LIGHTMAPM
}


