#include <conio.h>
#include <stdlib.h>
#include <math.h>

#include <pr.h>
#include <prgui.h>

#ifdef __3DFX__
#include <pr3dfx.h>
#include <glide.h>
#endif

#ifdef WIN32
#include <winutil.h>
#endif

#include <prdev.h>

/* Power Render Indoor Example

   Description:  Demonstrates the new planar collision detection
                 routines by moving around in a room.
                 This example uses precalculated RGB lighting so it
                 will not run in 256 bit VGA or SVGA mode.

   Controls:
      Up arrow or right mouse button - Walk forward
      Down arrow - Walk backward
      Space - Jump
      ESC - quits

   Suggestions for things that could be added:

   - Check for collisions above you when jumping

   - Make emitters that come out of the gun.  Collision detection
     between emitters and walls can be done by creating rays
     from the emitter's current and previous locations (stored in the
     emitter structure).

   - Multiple rooms.. Each room would be a PRO file with an associated
     collision mesh.  Special boxes or polytopes would be set up at the
     room entrances so when you move into them, it signals you have entered
     a new room.  This allows you to subdivide an entire level and only need
     to check for collisions on the polytopes of the current room.
     Each room could have a list of other rooms that could be visible from
     there, and only transform and render those ones. (PVS)
     When the level is complete you could package up all the data files into
     a single file using wgtlib.

   - Moving platforms and lifts, keyframed moving parts...  You can reconstruct
     the list of polytopes at any time.  This will allow you to keyframe
     or transform individual objects in the scene and collision mesh and rebuild
     the polytope list from the transformed version.

   - Player's position related to moving parts... If a player is standing on
     a moving platform, move the player along with it.

   - Actions or events for switches and moving parts... Signals called when the
     player walks into certain kinds of polytopes, which activate a keyframed
     animation or moving a part.  You could also create pushable objects...
     if the polytope is movable and you collide with it, move it in the direction
     the player was going and make sure it doesn't hit another wall.

   - Addition of monsters/enemies using either polytopes or spherical collision
     detection between them and player
*/

//#define PLACE_GUN
                                             
PR_VIEWPORT viewport;                  /* Our viewport structure */
PR_CAMERA *camera;                     /* One camera */
PR_OBJECT *object_shape;               /* The shape loaded */
PR_OBJECT *collision_shape;            /* The collision mesh */
PR_ENTITY *collision_entity;           /* The collision mesh */
PR_ENTITY *entity;                     /* A single entity of the shape */
PR_DWORD animation_count;


#define MAXGUN 1
PR_OBJECT *gun_shapes[MAXGUN];
PR_ENTITY *gun_entity;

PR_CAMERA *gun_camera;                     /* One camera */


PR_DWORD vwidth, vheight;

#define MAX_POLYTOPES 64
PR_POLYTOPE *clist[MAX_POLYTOPES];
PR_DWORD NumPolytopes;

#define PLAYERHEIGHT 300
#define MAXSLOPE 100
#define GRAVITY 2

PR_POINT playerlocation;

PR_REAL velocity = 0;

void animation_timer (void)
{
 animation_count++;
}


void CreateGunObjects (void)
{
  gun_shapes[0] = PR_LoadPRO ("gun.pro", LOAD_NORMAL);
  gun_entity = PR_CreateEntity (gun_shapes[0], "gun");
  PR_ScaleEntity (gun_entity, 0.03353f, 0.03353f, 0.03353f);
  PR_MoveEntity (gun_entity, -60, -90, -830);

  /* Instead of rotating the gun to match the camera, I made a new
     camera which looks at the gun entity from a fixed position.

     Different guns could have different fixed positions so they
     are on the left or right sides. 
  */

  gun_camera = PR_AllocCamera ();          /* Initialize the camera */
  PR_InitializeCamera (gun_camera);

  PR_PositionCameraSource (gun_camera, 0, 0, -1000);
  PR_PositionCameraTarget (gun_camera, 0, 0, 0);
  PR_SetCameraMode (gun_camera, CAMFLAG_AIM_TARGET);
  gun_camera->fov = 1.5f / 3.0f * 3.1415f;
}


void DrawGun (void)
{
  PR_SetActiveCamera (gun_camera);
  PR_TransformEntity (gun_entity);
  PR_RenderEntity (gun_entity);


/* This was used to position and scale the gun */
#ifdef PLACE_GUN
  if (kbdon[KEY_Q])
     PR_MoveEntity (gun_entity, -10, 0, 0);
  if (kbdon[KEY_W])
     PR_MoveEntity (gun_entity, 10, 0, 0);
  if (kbdon[KEY_A])
     PR_MoveEntity (gun_entity, 0, -10, 0);
  if (kbdon[KEY_S])
     PR_MoveEntity (gun_entity, 0, 10, 0);
  if (kbdon[KEY_Z])
     PR_MoveEntity (gun_entity, 0, 0, -10);
  if (kbdon[KEY_X])
     PR_MoveEntity (gun_entity, 0, 0, 10);
  if (kbdon[KEY_E])
     PR_ScaleEntity (gun_entity, 0.95, 0.95, 0.95);
  if (kbdon[KEY_D])
     PR_ScaleEntity (gun_entity, 1.1, 1.1, 1.1);

  PRGUI_printf (0, 0, "%f %f %f %f", gun_entity->orientation.location.x,
  gun_entity->orientation.location.y,
  gun_entity->orientation.location.z,
  gun_entity->orientation.scale.x);
#endif
}




void ClearCollisionList (void)
{
PR_DWORD i;

  NumPolytopes = 0;

  for (i = 0; i < MAX_POLYTOPES; i++)
    clist[i] = NULL;
}


void AddEntityToCollisionList (PR_ENTITY *ent)
/* Converts all segments of an entity into polytopes and adds them to
   the master polytope list */
{
PR_DWORD i;
PR_POLYTOPE *tope;

  for (i = 0; i < ent->shape->num_segments; i++)
    {
     tope = PRCL_CreatePolytope (ent->shape, i);
     if (tope != NULL)
       {
        clist[NumPolytopes++] = tope;

        /* Here you could also initialize the polytope's type and ID based
           on the name of the segment */
       }
    }
}


void SlideAlongWall (PR_POINT *start, PR_POINT *end, PR_DWORD topenum, PR_RAY *ray,
                PR_REAL t, PR_DWORD planebits)
{
PR_DWORD plane;
PR_DWORD planemask;
PR_REAL dist;

  planemask = 1;
  for (plane = 0; plane < clist[topenum]->numplanes; plane++)
    {
     if (planebits & planemask)
       {
        dist = PRCL_DistanceToPlane (end, &clist[topenum]->planelist[plane]);
        dist = abs(dist) + 1;
        end->x += clist[topenum]->planelist[plane].normal.x * dist;
        end->y += clist[topenum]->planelist[plane].normal.y * dist;
        end->z += clist[topenum]->planelist[plane].normal.z * dist;
       }
     planemask <<= 1;
    }

  start->x = end->x;
  start->y = end->y;
  start->z = end->z;
}


void CheckPlayerCollision (PR_POINT *start, PR_POINT *direction, PR_REAL speed)
{
PR_DWORD i, res;

PR_POINT end, stepstart, stepend;
PR_DWORD planebits;

PR_RAY rayA;
PR_RAY rayB;
PR_RAY rayC;
PR_REAL t;
PR_DWORD adjust = 0, foundfloor, hitwall = 0;
PR_REAL gravdist;

  /* Ray A */
  end.x = start->x + direction->x * speed;
  end.y = start->y;
  end.z = start->z + direction->z * speed;

  res = PRCL_RayFromPoints (&rayA, start, &end);
  if (res != PRCL_FALSE)
    {
     for (i = 0; i < NumPolytopes; i++)
       {
        res = PRCL_RayIntersectsPolytope (clist[i], &rayA, &t, &planebits);
        if (res & PRCL_TRUE)
          {
           hitwall = TRUE;

           stepstart.x = rayA.start.x;
           stepstart.y = rayA.start.y + MAXSLOPE;
           stepstart.z = rayA.start.z;

           stepend.x = rayA.end.x;
           stepend.y = rayA.end.y + MAXSLOPE;
           stepend.z = rayA.end.z;

           res = PRCL_RayFromPoints (&rayB, &stepstart, &stepend);
           if (res == PRCL_FALSE)
             return;
           res = PRCL_RayIntersectsPolytope (clist[i], &rayB, &t, &planebits);
           if (!res)
             adjust = TRUE;
           else if (planebits > 0)
             SlideAlongWall (start, &end, i, &rayA, t, planebits);
          }
       }
    }

  if (!hitwall)
    {
     start->x = end.x;
     start->y = end.y;
     start->z = end.z;
    }

  if (adjust)
    {
     stepstart = stepend;

     foundfloor = FALSE;
     gravdist = 0;

     start->x = stepend.x;
     start->y = stepend.y - MAXSLOPE;
     start->z = stepend.z;

     /* This should be optimized */
     while ((foundfloor == FALSE) && (gravdist < MAXSLOPE))
       {
        gravdist += GRAVITY;
        stepend.y -= GRAVITY;
        res = PRCL_RayFromPoints (&rayC, &stepstart, &stepend);
        if (res == PRCL_FALSE)
          return;
                 
        for (i = 0; i < NumPolytopes; i++)
          {
           res = PRCL_RayIntersectsPolytope (clist[i], &rayC, &t, &planebits);
           if (res & PRCL_TRUE)
             {
              PRCL_RayClip (&rayC, &rayC, t);
              foundfloor = TRUE;
              velocity = 0;

              start->x = rayC.end.x;
              start->y = rayC.end.y;
              start->z = rayC.end.z;
              break;
             }
          }
       }
    }
  else
    {
     end.x = start->x;
     end.y = start->y;
     end.z = start->z;

     velocity += GRAVITY;

     end.y -= velocity;
     PRCL_RayFromPoints (&rayA, start, &end);

     for (i = 0; i < NumPolytopes; i++)
       {
        res = PRCL_RayIntersectsPolytope (clist[i], &rayA, &t, &planebits);
        if (res & PRCL_TRUE)
          {
           PRCL_RayClip (&rayA, &rayA, t);
           velocity = 0;
          }
       }

     start->x = rayA.end.x;
     start->y = rayA.end.y + 1.0f;
     start->z = rayA.end.z;
    }
}






void UserInput (void)
{
PR_POINT dir;

  PR_ReadInput ();

  if ((kbdon[KEY_SPACE]) && (velocity == 0))
    velocity = -50;

  PR_CameraDirection (camera);

  dir.x = -camera->direction.x;
  dir.y = -camera->direction.y;
  dir.z = -camera->direction.z;
  CheckPlayerCollision (&playerlocation, &dir, currentspeed);
  camera->source.x = playerlocation.x;
  camera->source.y = playerlocation.y + PLAYERHEIGHT;
  camera->source.z = playerlocation.z;
  PR_CameraDirection (camera);


  PR_SetActiveCamera (camera);
}


void LoadCollisionMesh (char *filename)
{
PR_DWORD segnum;

  collision_shape = PR_LoadPRO (filename, LOAD_NORMAL);
  collision_entity = PR_CreateEntity (collision_shape, "collision mesh");

  for (segnum = 0; segnum < collision_shape->num_segments; segnum++)
    collision_shape->segment_list[segnum].flags |= FLAG_ALWAYS_VISIBLE;
  
  PR_MatrixIdentity (PR_ViewMatrix);
  PR_TransformEntity (collision_entity);
  ClearCollisionList ();
  AddEntityToCollisionList (collision_entity);
}


void main (void)
{
PR_DWORD move_count;

  ChooseDevice ();

  PR_Initialize (1000);                /* Maximum 1000 triangles per frame */

  if ((device == DEVICE_VGA) || (device == DEVICE_SVGA))
    {
     #ifndef WIN32
      wsetmode (3);                        /* Go back to text mode */
     #endif
     PR_FatalError ("This example requires a 3D card", "INDOOR");
    }

  InitializeDevices ();

  PR_AllocMaterials (32);              /* Allocate the materials, */
  PR_AllocTextures (32);               /* textures, */
  PR_AllocShadeTables (2);             /* and shade tables */

  vwidth = PR_VideoModes[0].width;     /* Initialize the video mode */
  vheight = PR_VideoModes[0].height;
  PR_SetMode (vwidth, vheight, 60);

  PR_OpenViewport (&viewport, 0, 0, vwidth-1, vheight-1, VIEW_PLAIN);
  PR_SetViewport (&viewport);          /* Open a full screen viewport */

  PRGUI_GoStartPath ();

  /* Load the object */
  object_shape = PR_LoadPRO ("room.pro", LOAD_NORMAL);
  /* Create the main entity */
  entity = PR_CreateEntity (object_shape, "TEST ENTITY");
  
  CreateGunObjects ();

  camera = PR_AllocCamera ();          /* Initialize the camera */
  PR_InitializeCamera (camera);

  playerlocation.x = 0;
  playerlocation.y = 1000;
  playerlocation.z = 0;

  PR_PositionCameraSource (camera, 0, 1000, 0);
  PR_PositionCameraTarget (camera, 0, 0, 0);
  PR_SetCameraMode (camera, CAMFLAG_ANGLE_BASED);
  camera->fov = 1.5f / 3.0f * 3.1415f;
  PR_SetActiveCamera (camera);

  /* We must have the camera set up before calling this, 
     since PR_TransformEntity requires one */
  LoadCollisionMesh ("cmesh.pro");

  installkbd ();

  animation_count = 0;
  winittimer ();
  wstarttimer (animation_timer, TICKS (60));

#ifndef WIN32
  minit ();
#endif

  PR_SetInputDevice (INPUT_MOUSE);
  PR_SetInputModel (MODEL_WALKING);
  PR_SetInputEntity (NO_ENTITY, NO_SEGMENT);
  PR_SetInputCamera (camera);

  TRANSLATE_SPEED    = 6;
  MAX_TRANSLATE_SPEED= 44;
  TRANSLATE_DRAG     = 2.5;

  PITCH_SPEED        =  5;
  MAX_PITCH_SPEED    =  20;
  PITCH_DRAG         =  1;

  YAW_SPEED          =  5;
  MAX_YAW_SPEED      =  10;
  YAW_DRAG           =  1;

  PR_OpenScreen (PR_BACKBUFFER);


  /* main program loop */
  while (!kbdon[KEY_ESC])
    {
     #ifdef WIN32
       UpdateMessages ();
     #endif

     #if defined (MSDD) || defined (WTDD) || defined (MSGL) || defined (WTGL)
       if (PR_LostSurface)
         PR_RestoreAll ();
     #endif

     PR_NewFrame ();                   

     PRGFX_SetColor (0);
     PRGFX_ClearScreen ();

     UserInput ();


     move_count = animation_count;
     animation_count = 0;

     PR_TransformEntity (entity);      
     PR_RenderEntity (entity);         

     DrawGun ();

     PR_RenderFrame ();

     PRGFX_SetTextForeground (PRGFX_MakeColor (63, 63, 63));
     PRGFX_SetTextTransparent (TEXTFG);

     PR_Flip (1);                      
    }

  wstoptimer ();
  wdonetimer ();

  uninstallkbd ();
  mdeinit ();

  PR_FreeCamera (camera);
  PR_FreeCamera (gun_camera);
  PR_CloseViewport (&viewport);

  /* Free all allocated memory */
  PR_FreeEntity (entity);
  PR_FreeObject (object_shape);

  PR_FreeObject (gun_shapes[0]);
  PR_DeleteAllShadeTables ();
  PR_DeleteAllTextures ();
  PR_DeleteAllMaterials ();

  PR_ShutDown ();

#ifndef WIN32
  wsetmode (3);                        /* Go back to text mode */
#endif
}

