/* Power Render Polytope Collision Interface */

#ifndef _PRCL_
#define _PRCL_

#ifdef __cplusplus
  extern "C" {
#endif


/*
  Plane definition

  For any point on the plane, point dot normal equals the offset of the plane.
  The plane subdivides space into two half space. A point is considered to be
  in the half space defined by the plane if:  p.normal <= offset
*/
typedef struct {
    PR_POINT normal;
    PR_REAL  offset;
} PR_PLANE;


/* Defines a convex polytope with any number of sides */
typedef struct {
  PR_DWORD numplanes;
  PR_PLANE *planelist;
  PR_DWORD type;        /* User defined value */
  PR_DWORD id;          /* User defined value */
} PR_POLYTOPE;


typedef struct {
 PR_POINT min, max;       /* Two corners */
 PR_DWORD type;           /* User defined value */
 PR_DWORD id;             /* User defined value */     
} PR_BOX;

typedef struct {
    PR_POINT  start;    /* Start point */
    PR_POINT  end;      /* End point */
    PR_POINT  dir;      /* Normalize direction vector */
    float     len;      /* Length of vector */
} PR_RAY;


#define PRCL_FALSE    0
#define PRCL_TRUE     1
#define PRCL_START_IN 2
#define PRCL_END_IN   4
#define PRCL_ALL_IN   8


#define PR_PointInHalfSpace(pt,pln) \
       (PR_VecDotProduct((pt), (pln)->normal) <= (pln)->offset)


PR_REAL PRCL_DistanceToPlane (PR_POINT *p, PR_PLANE *plane);
/* -----------------------------------------------------------------------
   Function:   PRCL_DistanceToPlane
   Purpose:    Returns the distance from a point to a plane
   Parameters: p - Point
               plane - Plane
   Return:     Distance from point to plane
   ----------------------------------------------------------------------- */


PR_DWORD PRCL_PlaneFromPoints (PR_PLANE *plane, PR_POINT *v1, PR_POINT *v2, PR_POINT *v3);
/* -----------------------------------------------------------------------
   Function:   PRCL_PlaneFromPoints
   Purpose:    Creates a plane given three noncolinear points.
   Parameters: plane - pointer to the plane to create
               v1, v2, v3 - any 3 noncolinear points
   Return:     Returns TRUE if the plane is valid
   ----------------------------------------------------------------------- */


PR_DWORD PRCL_RayIntersectsPlane (PR_RAY *ray, PR_PLANE *plane, PR_REAL *t);
/* -----------------------------------------------------------------------
   Function:   PRCL_RayIntersectPlane
   Purpose:    Determines if and where a ray intersects a plane
   Parameters: ray - pointer to the ray
               plane - pointer to the plane
               t - pointer to the intersection parameter
   Return:     Returns PRCL_IS_TRUE if the ray intersects, PRCL_IS_FALSE otherwise
   ----------------------------------------------------------------------- */


PR_POLYTOPE *PRCL_AllocPolytope (PR_DWORD numplanes);
/* -----------------------------------------------------------------------
   Function:   PRCL_AllocPolytope
   Purpose:    Allocates a polytope with a given number of planes
   Parameters: numplanes - Number of planes in the polytope
   Return:     Pointer to the new polytope
   ----------------------------------------------------------------------- */


PR_POLYTOPE *PRCL_CreatePolytope (PR_OBJECT *shape, PR_DWORD segnum);
/* -----------------------------------------------------------------------
   Function:   PRCL_CreatePolytope
   Purpose:    Converts any segment into a polytope
               Only unique planes are stored in the polytope
               Maximum number of unique planes defining the polytope is 128
   Parameters: shape - Pointer to the object
               segnum - Segment number to convert to a polytope
   Return:     Pointer to the new polytope
   ----------------------------------------------------------------------- */


void PRCL_TransformPlane (PR_PLANE *src, PR_PLANE *dst, PR_MATRIX m);
/* -----------------------------------------------------------------------
   Function:   PRCL_TransformPlane
   Purpose:    Transforms a plane by an orthogonal matrix
   Parameters: src - Pointer to the original plane
               dst - Pointer to the transformed plane
               m   - Orthogonal matrix
   Return:     None
   ----------------------------------------------------------------------- */


void PRCL_TransformPolytope (PR_POLYTOPE *src, PR_POLYTOPE *dst, PR_MATRIX m);
/* -----------------------------------------------------------------------
   Function:   PRCL_TransformPolytope
   Purpose:    Transforms a polytope by an orthogonal matrix
   Parameters: src - Pointer to the original polytope
               dst - Pointer to the transformed polytope
               m   - Orthogonal matrix
   Return:     None
   ----------------------------------------------------------------------- */


PR_DWORD PRCL_RayFromPoints (PR_RAY *ray, PR_POINT *p1, PR_POINT *p2);
/* -----------------------------------------------------------------------
   Function:   PRCL_RayFromPoints
   Purpose:    Creates a ray given start (p1) and end (p2) points
   Parameters: ray - Pointer to the new ray
               p1 - pointer to the start location
               p2 - pointer to the end location
   Return:     PRCL_TRUE if the ray has a length greater than 0
   ----------------------------------------------------------------------- */


void PRCL_RayClip (PR_RAY *src, PR_RAY *dst, PR_REAL t);
/* -----------------------------------------------------------------------
   Function:   PRCL_RayClip
   Purpose:    Clips a ray 
   Parameters: src - Pointer to the original ray
               dst - Pointer to the clipped ray
               t - intersection parameter
   Return:     None
   ----------------------------------------------------------------------- */


PR_DWORD PRCL_RayIntersectsBox (PR_BOX *box, PR_RAY *ray, PR_REAL *t);
/* -----------------------------------------------------------------------
   Function:   PRCL_RayIntersectsBox
   Purpose:    Find if and where a ray intersects a box
               The nearest point of intersection is returned in t
   Parameters: box - Pointer to the box
               ray - Pointer to the ray
               t - Pointer to the intersection parameter
   Return:     Intersection status
   ----------------------------------------------------------------------- */



PR_DWORD PRCL_RayIntersectsPolytope (PR_POLYTOPE *tope, PR_RAY *ray, PR_REAL *t, PR_DWORD *planebits);
/* -----------------------------------------------------------------------
   Function:   PRCL_RayIntersectsPolytope
   Purpose:    Find if and where a ray intersects a polytope
               Returns the nearest point of intersection in t
   Parameters: tope - Pointer to the polytope
               ray - Pointer to the ray
               t - Pointer to the intersection parameter
               planebits - Bits telling which planes intersect the ray
   Return:     Intersection status
   ----------------------------------------------------------------------- */


PR_DWORD PRCL_RayIntersectsTriangle (PR_RAY *ray, PR_VERTEX *v0,
                        PR_VERTEX *v1, PR_VERTEX *v2, PR_REAL *t);
/* -----------------------------------------------------------------------
   Function:   PRCL_RayIntersectsTriangle
   Purpose:    Find if and where a ray intersects a triangle
               Returns the point of intersection in t
   Parameters: ray - Pointer to the ray
               v0, v1, v2 - Pointer to the vertices
               t - Pointer to the intersection parameter
   Return:     Intersection status
   ----------------------------------------------------------------------- */

#ifdef __cplusplus
}
#endif

#endif

