#ifndef _PRPARTICLE_            
#define _PRPARTICLE_           /* Mark this header already included */

#ifdef __cplusplus
 extern "C" {
#endif


/* Particle Flags */
#define PARTICLE_MOVES         1   /* Moves each frame */
#define PARTICLE_HAS_GRAVITY   2   /* Is affected by gravity */
#define PARTICLE_ALPHA_FADES   4   /* Alpha fades out */
#define PARTICLE_ONCE          8   /* Only shown once */
#define PARTICLE_TIME_LIMIT   16   /* Dies after a certain number of frames */
#define PARTICLE_ANIMATED     32   /* Uses animated objects */
#define PARTICLE_LOOP_ANIM    64   /* Looped animation */
#define PARTICLE_SCALES      128   /* Changes scale each frame */
#define PARTICLE_HAS_DRAG    256   /* Velocity slows down */
#define PARTICLE_CAN_COLLIDE 512   /* Collision detection required */


typedef struct {
PR_UWORD Index;

PR_DWORD Flags;

PR_POINT Location;          /* Current location, after rotation applied */
PR_POINT LastLocation;      /* Last origin */    
PR_POINT Velocity;          /* Current velocity */

PR_REAL  Gravity;           /* Added to y velocity each frame */
PR_REAL  Drag;              /* Multiplied by X and Z velocity each frame */

PR_WORD  InitialAlpha;      /* Initial alpha value */
PR_WORD  CurrentAlpha;      /* Current alpha value */
PR_WORD  AlphaDelta;        /* Added to alpha each update */

PR_WORD  TimeCount;         /* Number of frames left before emitter dies */
PR_WORD  MaxTime;           /* Lifespan, in frames */

PR_WORD  StartObject;       /* For animation */
PR_WORD  EndObject;       
PR_WORD  CurrentObject;
PR_WORD  UpdateCount;       /* Number of frames till next update of animation */
PR_WORD  NextUpdate;        /* Number of frames between updates */

PR_REAL  InitialScale;      /* Initial Scale */
PR_REAL  CurrentScale;      /* Current Scale */
PR_REAL  ScaleDelta;        /* Added to scale each frame */

void (*CollideProc)(void *particle);  /* Routine for collision detection */
void (*DrawProc)(void *particle);     /* Routine for drawing the particle */

PR_ENTITY *Entity;

PR_DWORD Previous;          /* Index to previous particle */
PR_DWORD Link;              /* Index to next particle */

} PR_PARTICLE;




/* Emitter flags */
#define EMITTER_MOVES              1  /* Moves each frame */
#define EMITTER_ROTATES            2  /* Rotates each frame */
#define EMITTER_HAS_GRAVITY        4  /* Is affected by gravity */
#define EMITTER_CAN_COLLIDE        8  /* Collision detection required */
#define EMITTER_COLLIDE_BOUNCES   16  /* Bounces if collision is found */
#define EMITTER_COLLIDE_STOPS     32  /* Disappears if collision is found */
#define EMITTER_COLLIDE_SPAWNS    64  /* Disappears and spawns a new emitter */
#define EMITTER_RANDOM_LOCATION  128  /* Emits particles with random distances from center */
#define EMITTER_RANDOM_VELOCITY  256  /* Emits particles with random velocities */
#define EMITTER_ONCE             512  /* Only emits particles for one update */
#define EMITTER_TIME_LIMIT      1024  /* Dies after a certain number of frames */
#define EMITTER_BOUNCE_LIMIT    2048  /* Dies after a certain number of bounces */


typedef struct {
PR_UWORD  Index;
PR_DWORD  Flags;

PR_POINT  Origin;           /* Current origin */
PR_POINT  Location;         /* Current location, after rotation applied */
PR_POINT  LastLocation;     /* Last location */

PR_POINT  Velocity;         /* Translational velocity */
PR_DWORD  MaxRandomSpeed;   /* Max Random Velocity */


PR_REAL   Gravity;          /* Added to y velocity each frame */
PR_POINT  BounceFactor;     /* Multiplied by velocity when bouncing */

PR_MATRIX CurrentRotation;  /* Allows for rotation about unique axis */
PR_MATRIX AngularVelocity;  /* Angular velocity */
PR_POINT  Distance;         /* Initial distance from center of rotation */

PR_DWORD  MaxRandomRadius;  /* Max Random distance from center */
PR_WORD   NumParticles;     /* Number of particles to emit each frame */

PR_WORD   UpdateCount;      /* Number of frames till next update */
PR_WORD   NextUpdate;       /* Number of frames between updates */

PR_WORD   TimeCount;        /* Number of frames left before emitter dies */
PR_WORD   MaxTime;          /* Lifespan, in frames */

PR_WORD   BounceCount;      /* Number of bounces left before emitter dies */
PR_WORD   MaxBounce;        /* Maximum number of bounces */

void (*SpawnProc)(void *emit);  /* Routine to spawn new emitters */
void (*CollideProc)(void *emit);  /* Routine for collision detection */

PR_PARTICLE *ParticleType;  /* Pointer to the particle type it creates */

PR_DWORD Previous;          /* Index to previous emitter */
PR_DWORD Link;              /* Index to next emitter */

} PR_EMITTER;



extern PR_EMITTER *EmitterList;
extern PR_DWORD MaxEmitter;
extern PR_DWORD EmittersUsed;

extern PR_PARTICLE *ParticleList;
extern PR_DWORD MaxParticle;
extern PR_DWORD ParticlesUsed;

extern PR_EMITTER *EmitterTypes;
extern PR_PARTICLE *ParticleTypes;

void PR_AllocParticles (PR_DWORD numparticles, PR_DWORD numtypes);
/* Allocates space for all of the particles */

void PR_CreateParticle (PR_PARTICLE *ParticleType, PR_POINT *location, PR_POINT *velocity);
/* Adds a new particle to the scene */

void PR_RemoveParticle (PR_UWORD partnum);
/* Returns the particle to the list of free nodes */

void PR_UpdateParticles (PR_DWORD ticks_passed);
/* Update all particles in the scene */


void PR_InitializeParticleType (PR_DWORD partnum);
void PR_SetParticleMovement (PR_DWORD partnum, PR_DWORD action);
void PR_SetParticleGravity (PR_DWORD partnum, PR_REAL gravity);
void PR_SetParticleDrag (PR_DWORD partnum, PR_REAL drag);
void PR_SetParticleAlpha (PR_DWORD partnum, PR_WORD initial, PR_DWORD delta);
void PR_SetParticleTimeLimit (PR_DWORD partnum, PR_DWORD numframes);
void PR_SetParticleAnimation (PR_DWORD partnum, PR_DWORD start, PR_DWORD end, PR_DWORD delay, PR_UCHAR loop);
void PR_SetParticleScale (PR_DWORD partnum, PR_REAL initial, PR_REAL delta);
void PR_SetParticleEntity (PR_DWORD partnum, PR_ENTITY *entity);
void PR_SetParticleCollisionProc (PR_DWORD partnum, void (*collideproc)(void *particle));
void PR_SetParticleDrawProc (PR_DWORD partnum, void (*drawproc)(void *particle));



void PR_AllocEmitters (PR_DWORD numemitters, PR_DWORD numtypes);
/* Allocates space for all of the emitters */

void PR_CreateEmitter (PR_DWORD type, PR_POINT *location, PR_POINT *direction);
/* Adds a new emitter to the scene */

void PR_RemoveEmitter (PR_UWORD emitnum);
/* Returns the emitter to the list of free nodes */

void PR_UpdateEmitters (PR_DWORD ticks_passed);
/* Update all emitters in the scene */


void PR_InitializeEmitterType (PR_DWORD emitnum);
void PR_SetEmitterMovement (PR_DWORD emitnum, PR_DWORD action);
void PR_SetEmitterRotation (PR_DWORD emitnum, PR_MATRIX BaseRotation, PR_MATRIX AngularVelocity, PR_POINT *distance);
void PR_SetEmitterGravity (PR_DWORD emitnum, PR_REAL gravity);
void PR_SetEmitterCollisionProc (PR_DWORD emitnum, void (*collideproc)(void *emit));
void PR_SetEmitterSpawnProc (PR_DWORD emitnum, void (*spawnproc)(void *emit));
void PR_SetEmitterCollisionAction (PR_DWORD emitnum, PR_DWORD action);
void PR_SetEmitterRandomDistance (PR_DWORD emitnum, PR_DWORD radius);
void PR_SetEmitterRandomVelocity (PR_DWORD emitnum, PR_DWORD speed);
void PR_SetEmitterTimeLimit (PR_DWORD emitnum, PR_DWORD numframes);
void PR_SetEmitterBounceParameters (PR_DWORD emitnum, PR_POINT *factor, PR_DWORD maxbounce);
void PR_SetEmitterDelay (PR_DWORD emitnum, PR_WORD delay);
void PR_SetEmitterNumParticles (PR_DWORD emitnum, PR_WORD number);
void PR_SetEmitterParticleType (PR_DWORD emitnum, PR_DWORD particletype);

extern PR_DWORD FirstAvailableParticle;
extern PR_DWORD FirstAvailableEmitter;

extern PR_DWORD FirstUsedParticle;
extern PR_DWORD FirstUsedEmitter;


#ifdef __cplusplus
}
#endif

#endif



